using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.Serialization;
using CommonClass;
using System.Reflection;

namespace CapitalBudgetingProblem
{
    [Serializable]
    public partial class CapitalBudgetingProblem : WorkingSession, ISerializable
    {
        
        protected override void InitializeComponentAndData()
        {
            InitializeComponent();
        }

        public CapitalBudgetingProblem()
        {
            InitializeComponentAndData();            
        }

        //public CapitalBudgetingProblem(Dictionary<string, object> extendData, IAlgorithm algorithm)
        //public CapitalBudgetingProblem(Dictionary<string, object> extendData, UserControl algorithmManager)
        public CapitalBudgetingProblem(Dictionary<string, object> extendData, ImplementedAlgorithmType algorithmType)
            : base(extendData)
        {
            switch (algorithmType)
            {
                case ImplementedAlgorithmType.GeneticAlgorithm:
                    algorithmManager = new AlgorithmManager();
                    break;
                case ImplementedAlgorithmType.SI:
                    break;
                case ImplementedAlgorithmType.error:
                    throw new Exception("Nepostojei algoritam!");
            }

            RegisterFinishedAlgorithmEvent();
            ShowAlgorithmManager();  
        }

        protected override DataTable SetInputFileTypes(DataTable data, string description, string name)
        {
            DataRow row = data.NewRow();

            row[description] = "Integer Programming Form";
            row[name] = "Ipf";

            data.Rows.Add(row);

            return data;
        }

        protected override DataTable SetOutputFileTypes(DataTable data, string description, string name)
        {
            DataRow row = data.NewRow();

            row[description] = "WriteResults";
            row[name] = "WriteResults";

            data.Rows.Add(row);

            return data;
        }
       
        protected override string SetupInputFileExtension()
        {
            return "Integer Programming Form (*.ipf) |*.ipf";
        }

        protected override CommonClass.IInputParser ProblemDefinition(string inputText, string selectedExtension)
        {
            rtbProblem.Text = inputText;
            rtbProblem.BringToFront();
            return DynamicalCall.GetInputParser(selectedExtension, inputText, Assembly.GetExecutingAssembly());
        }

        protected override IOutputParser SetOutputParser(string outputParserType)
        {
            return DynamicalCall.GetOutputParser(outputParserType, Assembly.GetExecutingAssembly());
        }

        protected override IInputParser SetInputParser(string inputParserType)
        {
            return DynamicalCall.GetInputParser(inputParserType, "", Assembly.GetExecutingAssembly());
        }

        protected override string SetInputText()
        {
            return rtbProblem.Text;
        }

        protected override string SetupOuputFileExtension()
        {
            return "Results | *.out";
        }

        protected override void MakeNewProblem()
        {
            this.rtbProblem.Clear();
            this.rtbProblem.BringToFront();
        }

        protected override string ProblemToString()
        {
            return rtbProblem.Text;
        }

        #region ISerializable Members
        public CapitalBudgetingProblem(SerializationInfo info, StreamingContext context)
            :base(info, context)
        {
            this.rtbProblem.Text = (string)info.GetValue("problemDefinition", typeof(string));

            RegisterFinishedAlgorithmEvent();
            ShowAlgorithmManager();
        }


        void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
        {
            GetObjectData(info, context);
            info.AddValue("problemDefinition", rtbProblem.Text);            
        }

        #endregion 

        private void rtbProblem_TextChanged(object sender, EventArgs e)
        {
            IsInputFileModify = true;
        }

    }
}

