using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using CommonClass;
using System.Globalization;

namespace CapitalBudgetingProblem.InputFileParsers
{
    public enum KeyWordsCBP
    {
        totalProfit,
        opportunityInstallMachins,
        limitationMachins,
        machineProducts,
        mathOperators
    };

    /// <summary>
    /// IntegerProgrammingForm
    /// </summary>
    [Serializable()]
    public class Ipf : CommonClass.IInputParser
    {

        private string _inputText;
        public string SetInputText
        {
            set { _inputText = value; }
        }

        private int _dimension;
        private double[] _totalProfit;
        private double[] _opportunityInstallMachins;
        private double[] _limitationMachins;
        private double[,] _machineProducts;
        /// <summary>
        /// 0-opportunityInstallMachin, 1-limitationMachins, 2... -machineProducts kako su zapisani u datoteci
        /// </summary>
        private string[] _mathOperators;
        public Dictionary<object, object> GetResult
        {
            get
            {
                Dictionary<object, object> result = new Dictionary<object, object>();
                result.Add(KeyWordsCBP.totalProfit, _totalProfit);
                result.Add(KeyWordsCBP.opportunityInstallMachins, _opportunityInstallMachins);
                result.Add(KeyWordsCBP.limitationMachins, _limitationMachins);
                result.Add(KeyWordsCBP.machineProducts, _machineProducts);
                result.Add(KeyWordsCBP.mathOperators, _mathOperators);

                return result;
            }
        }

        public Ipf(string inputText)
        {
            _inputText = inputText;
        }

        public Ipf(StreamReader inputFile)
        {
            _inputText = inputFile.ReadToEnd();            
        }

        private void SetDimension(string line)
        {
            string[] temp = line.Split(' ');

            try
            {
                _dimension = int.Parse(temp[1]);
            }
            catch
            {
                throw new ParserException("Dimenzija mora biti cijeli broj veci od nule!");
            }
        }

        private void SetVar(string var, out int dim, out double p)
        {
            dim = 0;
            p = 0;
        }

        private void SetTotalProfit(string line)
        {
            int i=0;
            _totalProfit = new double[_dimension];
            NumberFormatInfo nfi = new NumberFormatInfo();
            line = line.Remove(0, 3);
            while (i < line.Length)
            {
                StringBuilder element = new StringBuilder();

                while (line[i] == ' ')
                    i++;

                if (line[i] == '+' || line[i] == '-')
                {
                    element.Append(line[i]);
                    i++;
                }

                while (i < line.Length && (line[i] != '+' && line[i] != '-'))
                {
                    if(line[i] != ' ')
                        element.Append(line[i]);

                    i++;
                }

                string[] temp = element.ToString().Split('x');                
                if (temp.Length != 2)
                    throw new ParserException("Krivo zadan total profit funkcija!");
                if (temp[0] == "+" || temp[0] == "-")
                    temp[0] += 1;

                int dim;
                double p;

                try
                {
                    dim = int.Parse(temp[1]);
                    p = Convert.ToDouble(temp[0], nfi);

                }
                catch
                {
                    throw new ParserException("Krivo zadan total profit funkcija!");
                }

                try
                {
                    _totalProfit[dim - 1] = p;

                }
                catch
                {
                    throw new ParserException("Indeks varijable u total profit funkciji je veci nego dimenzija problema");
                }
            }
        }

        private void SetOpportunityInstallMachins(string line)
        {
            int i = 0;
            _opportunityInstallMachins = new double[_dimension + 1];
            NumberFormatInfo nfi = new NumberFormatInfo();
            line = line.Remove(0, 1);
            while (i < line.Length)
            {
                StringBuilder element = new StringBuilder();

                while (line[i] == ' ')
                    i++;

                if (line[i] == '<' || line[i] == '>' || line[i] == '=')
                {
                    element.Append(line[i]);
                    i++;
                    element.Append(line[i]);
                    i++;
                    _mathOperators[0] = element.ToString();
                    while (line[i] == ' ')
                        i++;
                    element.Length = 0;
                    while (i < line.Length)
                    {
                        if(line[i] != ' ')
                            element.Append(line[i]);
                        i++;
                    }

                    try
                    {

                        _opportunityInstallMachins[_opportunityInstallMachins.Length - 1] = Convert.ToDouble(element.ToString(), nfi);
                    }
                    catch
                    {
                        throw new ParserException("Krivo zadana opportunity Install Machins funkcija!");
                    }
                    continue;
                }

                if (line[i] == '+' || line[i] == '-')
                {
                    element.Append(line[i]);
                    i++;
                }

                while (i < line.Length && (line[i] != '+' && line[i] != '-') && line[i] != '<' && line[i] != '>' && line[i] != '=')
                {
                    if (line[i] != ' ')
                        element.Append(line[i]);

                    i++;
                }

                string[] temp = element.ToString().Split('x');
                if (temp.Length != 2)
                    throw new ParserException("Krivo zadana opportunity Install Machins funkcija!");
                if (temp[0] == "+" || temp[0] == "-")
                    temp[0] += 1;

                int dim;
                double p;

                try
                {
                    dim = int.Parse(temp[1]);
                    p = Convert.ToDouble(temp[0], nfi);

                }
                catch
                {
                    throw new ParserException("Krivo zadan opportunity Install Machins funkcija!");
                }

                try
                {
                    _opportunityInstallMachins[dim - 1] = p;

                }
                catch
                {
                    throw new ParserException("Indeks varijable u opportunity Install Machins funkciji je veci nego dimenzija problema");
                }
            }

        }

        private void SetLimitationMachins(string line)
        {
            int i = 0;
            _limitationMachins = new double[_dimension + 1];            
            NumberFormatInfo nfi = new NumberFormatInfo();
            line = line.Remove(0, 1);
            while (i < line.Length)
            {
                StringBuilder element = new StringBuilder();

                while (line[i] == ' ')
                    i++;

                if (line[i] == '<' || line[i] == '>' || line[i] == '=')
                {
                    element.Append(line[i]);
                    i++;
                    element.Append(line[i]);
                    i++;
                    _mathOperators[1] = element.ToString();
                    while (line[i] == ' ')
                        i++;
                    element.Length = 0;
                    while (i < line.Length)
                    {
                        if(line[i] != ' ')
                            element.Append(line[i]);
                        i++;
                    }

                    try
                    {

                        _limitationMachins[_limitationMachins.Length - 1] = Convert.ToDouble(element.ToString(), nfi);
                    }
                    catch
                    {
                        throw new ParserException("Krivo zadana Limitation Machins funkcija!");
                    }
                    continue;
                }

                if (line[i] == '+' || line[i] == '-')
                {
                    element.Append(line[i]);
                    i++;
                }

                while (i < line.Length && (line[i] != '+' && line[i] != '-') && line[i] != '<' && line[i] != '>' && line[i] != '=')
                {
                    if (line[i] != ' ')
                        element.Append(line[i]);

                    i++;
                }

                string[] temp = element.ToString().Split('x');
                if (temp.Length != 2)
                    throw new ParserException("Krivo zadana Limitation Machins funkcija!");
                if (temp[0] == "+" || temp[0] == "-")
                    temp[0] += 1;

                int dim;
                double p;

                try
                {
                    dim = int.Parse(temp[1]);
                    p = Convert.ToDouble(temp[0], nfi);

                }
                catch
                {
                    throw new ParserException("Krivo zadana Limitation Machins funkcija!");
                }

                try
                {
                    _limitationMachins[dim - 1] = p;

                }
                catch
                {
                    throw new ParserException("Indeks varijable u Limitation Machins funkciji je veci nego dimenzija problema");
                }
            }

        }

        int numConstraint = 2;
        private double[] SetMachineProducts(string line)
        {
            int i = 0;
            double[] constraint = new double[_dimension+1];
            //for (i = 0; i < constraint.Length; i++)
            //    constraint[i] = 0;

            NumberFormatInfo nfi = new NumberFormatInfo();

            while (i < line.Length)
            {
                StringBuilder element = new StringBuilder();

                while (line[i] == ' ')
                    i++;

                if (line[i] == '<' || line[i] == '>' || line[i] == '=')
                {
                    element.Append(line[i]);
                    i++;
                    element.Append(line[i]);
                    i++;
                    _mathOperators[numConstraint++] = element.ToString();
                    while (line[i] == ' ')
                        i++;
                    element.Length = 0;
                    while (i < line.Length)
                    {
                        if (line[i] != ' ')
                            element.Append(line[i]);
                        i++;
                    }

                    try
                    {

                        constraint[constraint.Length - 1] = Convert.ToDouble(element.ToString(), nfi);
                    }
                    catch
                    {
                        throw new ParserException("Krivo zadana jedna od Machine Products funkcija!");
                    }
                    continue;
                }

                if (line[i] == '+' || line[i] == '-')
                {
                    element.Append(line[i]);
                    i++;
                }

                while (i < line.Length && (line[i] != '+' && line[i] != '-') && line[i] != '<' && line[i] != '>' && line[i] != '=')
                {
                    if (line[i] != ' ')
                        element.Append(line[i]);

                    i++;
                }

                string[] temp = element.ToString().Split('x');
                if (temp.Length != 2)
                    throw new ParserException("Krivo zadana jedna od Machine Products funkcija!");
                if (temp[0] == "+" || temp[0] == "-")
                    temp[0] += 1;

                int dim;
                double p;

                try
                {
                    dim = int.Parse(temp[1]);
                    p = Convert.ToDouble(temp[0], nfi);

                }
                catch
                {
                    throw new ParserException("Krivo zadana jedna od Machine Products funkcija!");
                }

                try
                {
                    constraint[dim - 1] = p;

                }
                catch
                {
                    throw new ParserException("Indeks varijable u Machine Products funkciji je veci nego dimenzija problema");
                }
            }

                return constraint;
        }

        public void Start()
        {
            string[] temp = _inputText.Split(new string[] { Environment.NewLine }, StringSplitOptions.RemoveEmptyEntries);
            if (temp.Length == 1)
            {
                _inputText = _inputText.Replace("\n", Environment.NewLine);
                temp = _inputText.Split(new string[] { Environment.NewLine }, StringSplitOptions.RemoveEmptyEntries);
            }

            _mathOperators = new string[temp.Length - 2];
            int lineNubmer = 0;
            numConstraint = 2;
            List<double[]> machineProducts = new List<double[]>();
            foreach (string line in temp)
            {
                lineNubmer++;
                try
                {
                    if (line.Contains("dim"))
                        SetDimension(line);
                    else if (line.Contains("max"))
                        SetTotalProfit(line);
                    else if (line[0] == 'a')
                        SetOpportunityInstallMachins(line);
                    else if (line[0] == 'b')
                        SetLimitationMachins(line);
                    else
                    {
                        machineProducts.Add(SetMachineProducts(line));
                    }
                }
                catch (ParserException e)
                {
                    throw new ParserException(string.Format("ERROR LINE:{0}{1}{2}", lineNubmer, Environment.NewLine, e.Message));
                }
            }

            _machineProducts = new double[temp.Length - 4, machineProducts.Count+1];            
            for (int i = 0; i < machineProducts.Count;i++ )
            {
                for (int j = 0; j < machineProducts[i].Length; j++)
                {
                    _machineProducts[i, j] = machineProducts[i][j];
                }
            }
        }
    }
}
