using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Runtime.Serialization;

namespace CommonClass
{
    public delegate void FinishedAlgorithmHandler();

    [Serializable]
    public partial class AlgorithmManager : UserControl, ISerializable
    {
        public event FinishedAlgorithmHandler FinishedAlgorithm;
        public void FireFinishedAlgorithm()
        {
            if(FinishedAlgorithm != null)
                FinishedAlgorithm();
        }

        protected Dictionary<object, object> _inputData = new Dictionary<object, object>();
        public Dictionary<object, object> InputData
        {
            get { return _inputData; }
            set { _inputData = value; }
        }

        protected Dictionary<object, object> _outputData = new Dictionary<object, object>();
        public Dictionary<object, object> OutputData
        {
            get 
            {
                //if (!this.DesignMode && _outputData.Count == 0)
                //    if (MessageBox.Show("Pokrenite algoritam kako bi dobili izlazne podatke!\n elite li pokrenuti algoritam?", "Upozorenje!", MessageBoxButtons.YesNo) == DialogResult.Yes)
                //        StartAlgorithm();

                return _outputData; 
            }
        }

        public AlgorithmManager()
        {
            InitializeComponent();
        }

        public virtual void StartAlgorithm() { throw new Exception("Implementirajte metodu >>StartAlgorithm()<< "); }
        public virtual void EndAlgorithm() { throw new Exception("Implementirajte metodu >>EndAlgorithm()<< "); }
        private void btnStartAlgorithm_Click(object sender, EventArgs e)
        {
            StartAlgorithm();
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            EndAlgorithm();
        }

        #region ISerializable Members

        public AlgorithmManager(SerializationInfo info, StreamingContext contex)            
        {
            InitializeComponent();
            _inputData = info.GetValue("inputData", typeof(Dictionary<object, object>)) as Dictionary<object, object>;
            _outputData = info.GetValue("outputData", typeof(Dictionary<object, object>)) as Dictionary<object, object>;
        }

        public void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("inputData", _inputData);
            info.AddValue("outputData", _outputData);
        }

        #endregion
    }
}
