using System;
using System.Collections;
using System.Text;
using System.Runtime.Serialization;

namespace LinearProgramming_SIMPLEX.GeneticAlgorithmClass
{
    [Serializable]
    class GenetskiAlgoritam : ISerializable
    {
        private int velicinaPopulacije;
        private double vjerojatnostMutacije;
        private int brojGeneracija;

        private static Vektor gornjaGranicaDimenzije;
        private static Vektor donjaGranicaDimenzije;
        private static int brojDecMjesta;
        private static int brojDimenzija;
        private bool max;

        private Random rand;

        private ArrayList populacija;

        public GenetskiAlgoritam(int inVelicinaPopulacije, double inVjerojatnostMutacije, int inBrojGeneracija, bool inMax)
        {
            this.velicinaPopulacije = inVelicinaPopulacije;
            this.vjerojatnostMutacije = inVjerojatnostMutacije;
            this.brojGeneracija = inBrojGeneracija;
            max = inMax;
            
            this.populacija = new ArrayList(velicinaPopulacije);
            rand = new Random((int)DateTime.Now.Ticks);
        }

        public static Vektor GornjaGranicaDimenzije
        {
            set
            {
                gornjaGranicaDimenzije = value;
            }
        }

        public static Vektor DonjaGranicaDimenzije
        {
            set
            {
                donjaGranicaDimenzije = value;
            }
        }

        public static int BrojDecMjesta
        {
            get
            {
                return brojDecMjesta;
            }
            set
            {
                brojDecMjesta = value;
            }
        }

        public static int BrojDimenzija
        {
            get
            {
                return brojDimenzija;
            }
            set
            {
                brojDimenzija = value;
            }
        }

        private uint BrojBinarnihMjesta(int i)
        {
            double broj = (gornjaGranicaDimenzije[i] - donjaGranicaDimenzije[i]) * Math.Pow(10, brojDecMjesta) + 1;
            return (uint)Math.Ceiling(Math.Log(broj, 2));
        }

        public static Binarni BinaranBroj(double x, uint brBinMjesta, int i)
        {
            uint broj = (uint)((x - donjaGranicaDimenzije[i]) * (Math.Pow(2, brBinMjesta) - 1));
            uint b = (uint)(broj / (gornjaGranicaDimenzije[i] - donjaGranicaDimenzije[i]));
            return new Binarni(b, brBinMjesta);
        }
        public static double BinarniUBroj(Binarni b, int i)
        {
            double first = b.Broj * (gornjaGranicaDimenzije[i] - donjaGranicaDimenzije[i]);
            return donjaGranicaDimenzije[i] + first / (Math.Pow(2, b.Velicina) - 1);
        }

        public void StvoriPopulaciju(Problem p)
        {
                        
            for (int i = 0; i < velicinaPopulacije; i++)
            {
                Vektor v = new Vektor(brojDimenzija);
                Kromosom k = new Kromosom(brojDimenzija, p);                
                do
                {
                    for (int j = 0; j < brojDimenzija; j++)
                    {
                        double t = rand.NextDouble();
                        double temp = t * (gornjaGranicaDimenzije[j] - donjaGranicaDimenzije[j]) + donjaGranicaDimenzije[j];
                        k[j] = BinaranBroj(temp, BrojBinarnihMjesta(j), j);
                        v[j] = temp;
                        k.RealniElementi = v;
                    }
                    
                } while (!p.Uvjeti(k.RealniElementi));
                
                k.IzracunajDobrotu();
                populacija.Add(k);
            }
        }

        private ArrayList Odaberi3Kromosoma()
        {
            ArrayList odabraniKromosomi = new ArrayList(3);
            int[] odabrani = new int[3];

            odabrani[0] = rand.Next(0, velicinaPopulacije);
            odabraniKromosomi.Add((Kromosom)populacija[odabrani[0]]);

            do
            {
                odabrani[1] = rand.Next(0, velicinaPopulacije);
            } while (odabrani[1] == odabrani[0]);
            odabraniKromosomi.Add((Kromosom)populacija[odabrani[1]]);

            do
            {
                odabrani[2] = rand.Next(0, velicinaPopulacije);
            } while (odabrani[2] == odabrani[1] || odabrani[2] == odabrani[0]);
            odabraniKromosomi.Add((Kromosom)populacija[odabrani[2]]);

            return odabraniKromosomi;
        }

        private void ObrisiNajlosijiKromosom(ArrayList odabraniKromosomi)
        {
            Kromosom najgoriKromosom = (Kromosom)odabraniKromosomi[0];

            if (max)
            {
                for (int i = 1; i < 3; i++)
                {
                    if (((Kromosom)odabraniKromosomi[i]).Dobrota < najgoriKromosom.Dobrota)
                    {
                        najgoriKromosom = (Kromosom)odabraniKromosomi[i];
                    }

                }
            }
            else
            {
                for (int i = 1; i < 3; i++)
                {
                    if (((Kromosom)odabraniKromosomi[i]).Dobrota > najgoriKromosom.Dobrota)
                    {
                        najgoriKromosom = (Kromosom)odabraniKromosomi[i];
                    }
                }
            }
            populacija.Remove(najgoriKromosom);
            odabraniKromosomi.Remove(najgoriKromosom);
        }

        private Kromosom Krizanje(ArrayList odabraniKromosomi, Problem p)
        {
            Kromosom prvi = (Kromosom)odabraniKromosomi[0];
            Kromosom drugi = (Kromosom)odabraniKromosomi[1];
            uint[] maxValue = new uint[brojDimenzija];
            for (int i = 0; i < brojDimenzija; i++)
            {
                maxValue[i] = (uint)Math.Pow(2, prvi[i].Velicina);
            }

            Vektor v = new Vektor(prvi.DimenzijaProblema);
            Kromosom noviKromosom = new Kromosom(prvi.DimenzijaProblema, prvi.OdabraniProblem);
            
            do
            {              
                for (int i = 0; i < brojDimenzija; i++)
                {
                    uint temp1 = prvi[i].Broj & drugi[i].Broj;
                    uint temp2 = prvi[i].Broj ^ drugi[i].Broj;
                    uint temp3 = temp2 & (uint)rand.Next((int)maxValue[i]);
                    noviKromosom[i] = new Binarni(temp1 | temp3, prvi[i].Velicina);
                    noviKromosom.RealniElementi[i] = GenetskiAlgoritam.BinarniUBroj(noviKromosom[i], i);                    
                    
                }                                                              
            } while (!p.Uvjeti(noviKromosom.RealniElementi));
            
            return noviKromosom;
        }

        private Vektor DohvatiMaximum()
        {
            Kromosom najboljiKromosom = (Kromosom)populacija[0];
            for (int i = 1; i < populacija.Count; i++)
            {
                if (((Kromosom)populacija[i]).Dobrota > najboljiKromosom.Dobrota)
                {
                    najboljiKromosom = (Kromosom)populacija[i];
                }
            }

            return najboljiKromosom.RealniElementi;            
        }

        private Vektor DohvatiMinimum()
        {
            Kromosom najlosijiKromosom = (Kromosom)populacija[0];
            for (int i = 0; i < populacija.Count; i++)
            {
                if (((Kromosom)populacija[i]).Dobrota < najlosijiKromosom.Dobrota)
                {
                    najlosijiKromosom = (Kromosom)populacija[i];
                }
            }
            return najlosijiKromosom.RealniElementi;
        }

        public Vektor PokreniGA(Problem p, System.Windows.Forms.RichTextBox status, bool ponavljanje)
        {
            StvoriPopulaciju(p);

            int brojac = 0;
            double dobrotaPrethodnog;

            if (max) dobrotaPrethodnog = 0;
            else dobrotaPrethodnog = double.MaxValue;

            do
            {                

                ArrayList odabraniKromosomi = Odaberi3Kromosoma();
                ObrisiNajlosijiKromosom(odabraniKromosomi);
                Kromosom noviKromosom = Krizanje(odabraniKromosomi, p);
                noviKromosom.Mutacija(vjerojatnostMutacije,rand);
                noviKromosom.IzracunajDobrotu();
                populacija.Add(noviKromosom);                

                if(max)
                {                    
                    Kromosom k = new Kromosom(DohvatiMaximum(), p);

                    if (!ponavljanje)
                    {
                        if (dobrotaPrethodnog < k.Dobrota)
                        {
                            status.AppendText("Generacaija " + brojac + " :: ");
                            for (int i = 0; i < p.DimenzijaProblema; i++)
                            {
                                status.AppendText("\n\t x" + (int)(i + 1) + " = " + k.RealniElementi[i]);
                            }
                            status.AppendText("\n\t Funkcija cilja = " + k.Dobrota + "\n");
                            status.ScrollToCaret();

                            dobrotaPrethodnog = k.Dobrota;
                        }
                    }
                    
                }
                else
                {
                    Kromosom k = new Kromosom(DohvatiMinimum(), p);

                    if (!ponavljanje)
                    {
                        if (dobrotaPrethodnog > k.Dobrota)
                        {
                            status.AppendText("Generacaija " + brojac + " :: ");
                            for (int i = 0; i < p.DimenzijaProblema; i++)
                            {
                                status.AppendText("\n\t x" + (int)(i + 1) + " = " + k.RealniElementi[i]);
                            }
                            status.AppendText("\n\t Funkcija cilja = " + k.Dobrota + "\n");
                            status.ScrollToCaret();

                            dobrotaPrethodnog = k.Dobrota;
                        }
                    }
                }
                
                brojac++;

            } while (brojac < brojGeneracija);

            if(max) return DohvatiMaximum();

            return DohvatiMinimum();
        }

        #region ISerializable Members

        public GenetskiAlgoritam(SerializationInfo info, StreamingContext contex)
        {
            velicinaPopulacije = (int)info.GetValue("VelicinaPopulacije", typeof(int));
            vjerojatnostMutacije = (double)info.GetValue("VjerojatnostMutacije", typeof(double));
            brojGeneracija = (int)info.GetValue("BrojGeneracija", typeof(int));
            gornjaGranicaDimenzije = info.GetValue("GornjaGranicaDimenzije", typeof(Vektor)) as Vektor;
            donjaGranicaDimenzije = info.GetValue("DonjaGranicaDimenzije", typeof(Vektor)) as Vektor;
            brojDecMjesta = (int)info.GetValue("BrojDecMjesta", typeof(int));
            brojDimenzija = (int)info.GetValue("BrojDimenzija", typeof(int));
            max = (bool)info.GetValue("Max", typeof(bool));

            this.populacija = new ArrayList(velicinaPopulacije);
            rand = new Random((int)DateTime.Now.Ticks);
        }

        public void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("VelicinaPopulacije", velicinaPopulacije);
            info.AddValue("VjerojatnostMutacije", vjerojatnostMutacije);
            info.AddValue("BrojGeneracija", brojGeneracija);
            info.AddValue("GornjaGranicaDimenzije", gornjaGranicaDimenzije);
            info.AddValue("DonjaGranicaDimenzije", donjaGranicaDimenzije);
            info.AddValue("BrojDecMjesta", brojDecMjesta);
            info.AddValue("BrojDimenzija", brojDimenzija);
            info.AddValue("Max", max);
        }

        #endregion
    }
}
