using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.Serialization;

namespace LinearProgramming_SIMPLEX.GeneticAlgorithmClass
{
    [Serializable]
    class GraniceProblema : ISerializable
    {
        private Vektor donjaGranica;
        private Vektor gornjaGranica;
        private Problem problem;
        private double preciznostGranica;

        public GraniceProblema(Problem inProblem, double inPreciznostGranica)
        {
            problem = inProblem;
            donjaGranica = new Vektor(problem.DimenzijaProblema);
            gornjaGranica = new Vektor(problem.DimenzijaProblema);
            preciznostGranica = inPreciznostGranica;
        }

        public Vektor DonjaGranica
        {
            get
            {
                return donjaGranica;
            }
        }

        public Vektor GornjaGranica
        {
            get
            {
                return gornjaGranica;
            }
        }


        private void odrediGornjeGranice(int dim, Vektor v)
        {
            while (true)
            {
                bool uvjet = false;
                for (double i = gornjaGranica[0]; i >= donjaGranica[0]; i -= preciznostGranica)
                {
                    v[0] = i;

                    Vektor pravi = new Vektor(v);
                    double tmp = pravi[dim];
                    pravi[dim] = pravi[problem.DimenzijaProblema - 1];
                    pravi[problem.DimenzijaProblema - 1] = tmp;

                    if (problem.Uvjeti(pravi))
                    {
                        uvjet = true;
                        break;
                    }
                }
                if (uvjet) break;
                int noviTrenutni = 0;
                
                while (noviTrenutni < problem.DimenzijaProblema)
                {
                    if (v[noviTrenutni] - preciznostGranica < donjaGranica[noviTrenutni])
                    {
                        v[noviTrenutni] = gornjaGranica[noviTrenutni];
                    }
                    else
                    {
                        v[noviTrenutni] -= preciznostGranica;                        
                        break;
                    }
                    noviTrenutni++;
                }

                if (v[problem.DimenzijaProblema - 1] - preciznostGranica < donjaGranica[problem.DimenzijaProblema - 1])
                {
                    v[problem.DimenzijaProblema - 1] = -1;
                    break;
                }
            }
        }

        private void odrediDonjeGranice(int dim, Vektor v)
        {
            while (true)
            {
                bool uvjet = false;
                for (double i = donjaGranica[0]; i <= gornjaGranica[0]; i += preciznostGranica)
                {
                    v[0] = i;

                    Vektor pravi = new Vektor(v);
                    double tmp = pravi[dim];
                    pravi[dim] = pravi[problem.DimenzijaProblema - 1];
                    pravi[problem.DimenzijaProblema - 1] = tmp;

                    if (problem.Uvjeti(pravi))
                    {
                        uvjet = true;
                        break;
                    }
                }
                if (uvjet) break;
                int noviTrenutni = 0;

                while (noviTrenutni < problem.DimenzijaProblema)
                {
                    if (v[noviTrenutni] + preciznostGranica > gornjaGranica[noviTrenutni])
                    {
                        v[noviTrenutni] = donjaGranica[noviTrenutni];
                    }
                    else
                    {
                        v[noviTrenutni] += preciznostGranica;
                        break;
                    }
                    noviTrenutni++;
                }
                if (v[problem.DimenzijaProblema - 1] + preciznostGranica > donjaGranica[problem.DimenzijaProblema - 1])
                {
                    v[problem.DimenzijaProblema - 1] = -1;
                    break;
                }
            }
        }       
        
        public void IzracunajGranice()
        {
            try
            {
                for (int i = 0; i < donjaGranica.Velicina; i++)
                {
                    donjaGranica[i] = 0.0;
                    gornjaGranica[i] = 0.0;
                }

                double[,] vektor = new double[problem.DimenzijaProblema, problem.DimenzijaProblema];

                double[,] rezultat = new double[problem.BrojUvjeta, problem.DimenzijaProblema];

                for (int i = 0; i < problem.DimenzijaProblema; i++)
                {
                    for (int j = 0; j < problem.DimenzijaProblema; j++)
                    {
                        if (i == j) vektor[i, j] = 1.0;
                        else vektor[i, j] = 0.0;
                    }
                }

                for (int i = 0; i < problem.BrojUvjeta; i++)
                {
                    for (int j = 0; j < problem.DimenzijaProblema; j++)
                    {
                        for (int k = 0; k < problem.DimenzijaProblema; k++)
                        {
                            rezultat[i, j] += problem.MatricaUvjeta[i, k] * vektor[k, j];
                        }
                    }
                }

                for (int i = 0; i < problem.BrojUvjeta; i++)
                {
                    for (int j = 0; j < problem.DimenzijaProblema; j++)
                    {
                        if (rezultat[i, j] != 0.0)
                        {
                            rezultat[i, j] = problem.MatricaUvjeta[i, problem.DimenzijaProblema] / rezultat[i, j];
                        }
                    }
                }

                for (int i = 0; i < problem.DimenzijaProblema; i++)
                {
                    for (int j = 0; j < problem.BrojUvjeta; j++)
                    {
                        if (rezultat[j, i] > gornjaGranica[i]) gornjaGranica[i] = rezultat[j, i];

                        if (rezultat[j, i] < donjaGranica[i] && rezultat[j, i] > 0.0) donjaGranica[i] = rezultat[j, i];
                    }
                }


                Vektor v = new Vektor(gornjaGranica);
                Vektor rez = new Vektor(problem.DimenzijaProblema);

                for (int i = 0; i < problem.DimenzijaProblema; i++)
                {
                    double dim = v[i];
                    v[i] = v[problem.DimenzijaProblema - 1];
                    v[problem.DimenzijaProblema - 1] = dim;

                    dim = gornjaGranica[i];
                    gornjaGranica[i] = gornjaGranica[problem.DimenzijaProblema - 1];
                    gornjaGranica[problem.DimenzijaProblema - 1] = dim;

                    
                    odrediGornjeGranice(i, v);


                    dim = v[i];
                    v[i] = v[problem.DimenzijaProblema - 1];
                    v[problem.DimenzijaProblema - 1] = dim;

                    dim = gornjaGranica[i];
                    gornjaGranica[i] = gornjaGranica[problem.DimenzijaProblema - 1];
                    gornjaGranica[problem.DimenzijaProblema - 1] = dim;

                    if (v[i] != -1)
                    {
                        rez[i] = v[i];
                        rez[i] += preciznostGranica;
                    }
                    else
                    {
                        rez[i] = gornjaGranica[i];
                    }


                    for (int k = 0; k < problem.DimenzijaProblema; k++)
                        v[k] = gornjaGranica[k];

                    v[i] = rez[i];
                }
                for (int i = 0; i < problem.DimenzijaProblema; i++)
                {
                    gornjaGranica[i] = rez[i];
                    v[i] = donjaGranica[i];
                }

                rez = null;
                rez = new Vektor(problem.DimenzijaProblema);

                for (int i = 0; i < problem.DimenzijaProblema; i++)
                {
                    double dim = v[i];
                    v[i] = v[problem.DimenzijaProblema - 1];
                    v[problem.DimenzijaProblema - 1] = dim;

                    dim = donjaGranica[i];
                    donjaGranica[i] = donjaGranica[problem.DimenzijaProblema - 1];
                    donjaGranica[problem.DimenzijaProblema - 1] = dim;

                    
                    odrediDonjeGranice(i, v);
                    


                    dim = v[i];
                    v[i] = v[problem.DimenzijaProblema - 1];
                    v[problem.DimenzijaProblema - 1] = dim;

                    dim = donjaGranica[i];
                    donjaGranica[i] = donjaGranica[problem.DimenzijaProblema - 1];
                    donjaGranica[problem.DimenzijaProblema - 1] = dim;

                    if (v[i] != -1)
                    {
                        rez[i] = v[i];
                    }
                    else
                    {
                        rez[i] = donjaGranica[i];
                    }


                    for (int k = 0; k < problem.DimenzijaProblema; k++)
                        v[k] = gornjaGranica[k];

                    v[i] = rez[i];
                }                

                for (int i = 0; i < problem.DimenzijaProblema; i++)
                    donjaGranica[i] = rez[i];


                for (int i = 0; i < problem.DimenzijaProblema; i++)
                {
                    if (gornjaGranica[i] - donjaGranica[i] < preciznostGranica)
                    {
                        if (gornjaGranica[i] == 0) gornjaGranica[i] += preciznostGranica;
                        donjaGranica[i] = 0;
                    }
                }                
            }
            catch (Exception e)
            {
                throw new Exception(e.Message);
            }
            
        }

        #region ISerializable Members

        public GraniceProblema(SerializationInfo info, StreamingContext contex)
        {
            donjaGranica = info.GetValue("DonjaGranica", typeof(Vektor)) as Vektor;
            gornjaGranica = info.GetValue("GornjaGranica", typeof(Vektor)) as Vektor;
            problem = info.GetValue("Problem", typeof(Problem)) as Problem;
            preciznostGranica = (double)info.GetValue("PreciznostGranica", typeof(double));            
        }

        public void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("DonjaGranica", donjaGranica);
            info.AddValue("GornjaGranica", gornjaGranica);
            info.AddValue("Problem", problem);
            info.AddValue("PreciznostGranica", preciznostGranica);
        }

        #endregion
    }
}
