using System;
using System.Collections.Generic;
using System.Text;

namespace LinearProgramming_SIMPLEX.GeneticAlgorithmClass
{
    class Kromosom
    {
        private Vektor realniElementi;
        private Binarni[] elementi;
        private int dimenzijaProblema;
        private double dobrota;
        private Problem problem;

        public Kromosom(int dimProblema, Problem p)
        {
            this.dimenzijaProblema = dimProblema;
            this.problem = p;
            elementi = new Binarni[dimenzijaProblema];
            realniElementi = new Vektor(dimProblema);
        }

        public Kromosom(Vektor v, Problem p)
        {
            this.dimenzijaProblema = v.Velicina;
            this.problem = p;
            elementi = new Binarni[p.DimenzijaProblema];
            realniElementi = v;
            this.IzracunajDobrotu();
        }

        public Kromosom(Kromosom k)
        {
            this.dimenzijaProblema = k.dimenzijaProblema;
            this.problem = k.problem;
            this.elementi = k.elementi;
            this.realniElementi = k.realniElementi;
            this.dobrota = k.dobrota;
        }

        public Vektor RealniElementi
        {
            get
            {
                return realniElementi;
            }
            set
            {
                realniElementi = value;
            }

        }

        public Binarni this[int index]
        {
            get
            {
                return this.elementi[index];
            }
            set
            {
                this.elementi[index] = value;
            }
        }

        public double Dobrota
        {
            get
            {
                return this.dobrota;
            }
        }

        public int DimenzijaProblema
        {
            get
            {
                return this.dimenzijaProblema;
            }
        }

        public Problem OdabraniProblem
        {
            get
            {
                return this.problem;
            }
        }

        public void IzracunajDobrotu()
        {
            this.dobrota = problem.IzracunajFunkCilja(this.realniElementi);
        }

        public void Mutacija(double vjerojatnost, Random rand)
        {
            int dimenzija = rand.Next(this.dimenzijaProblema);
            int duljina = (int)this[dimenzija].Velicina;
            int ukupnoBitova = duljina * dimenzijaProblema;

            double vjerojatnostMutacijeKromosoma = 1 - Math.Pow(1 - vjerojatnost, ukupnoBitova);
            if (rand.NextDouble() <= vjerojatnostMutacijeKromosoma)
            {               
                int bit = rand.Next((int)this[dimenzija].Velicina);

                if (this[dimenzija][bit] == 1)
                {
                    this[dimenzija][bit] = 0;
                    this.RealniElementi[dimenzija] = GenetskiAlgoritam.BinarniUBroj(this[dimenzija], dimenzija);

                    if (!problem.Uvjeti(this.realniElementi))
                    {
                        this[dimenzija][bit] = 1;
                        this.RealniElementi[dimenzija] = GenetskiAlgoritam.BinarniUBroj(this[dimenzija], dimenzija);

                    }
                }
                else
                {
                    this[dimenzija][bit] = 1;
                    this.RealniElementi[dimenzija] = GenetskiAlgoritam.BinarniUBroj(this[dimenzija], dimenzija);

                    if (!problem.Uvjeti(this.realniElementi))
                    {
                        this[dimenzija][bit] = 0;
                        this.RealniElementi[dimenzija] = GenetskiAlgoritam.BinarniUBroj(this[dimenzija], dimenzija);

                    }
                }
            }
        }
    }
}
