using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

namespace LinearProgramming_SIMPLEX.GeneticAlgorithmClass
{
    class Parser
    {
        private string textProblema;
        private int dimenzijaProblema;
        private int brojUvjeta;
        private double[] matricaCilja;
        private double[,] matricaUvjeta;
        private string[] operatorOdnosa;
        private bool max;

        public Parser(string inTextProblema)
        {
            this.textProblema = inTextProblema;
        }


        public int DimenzijaProblema
        {
            get
            {
                return dimenzijaProblema;
            }
        }

        public int BrojUvjeta
        {
            get
            {
                return brojUvjeta;
            }
        }

        public double[] MatricaCilja
        {
            get
            {
                return matricaCilja;
            }
        }

        public double[,] MatricaUvjeta
        {
            get
            {
                return matricaUvjeta;
            }
        }

        public string[] MatricaOdnosa
        {
            get
            {
                return operatorOdnosa;
            }
        }

        public bool Max
        {
            get
            {
                return max;
            }
        }

        /// <summary>
        /// Parsira se ulazni niz koji sadrzi dim a predstavlja dimenziju problema
        /// </summary>
        /// <param name="ulaz"></param>
        private void Dimenzija(string ulaz)
        {            
            StringBuilder sb = new StringBuilder();
            
            for (int i = 0; i < ulaz.Length; i++)
            {
                if (char.IsDigit(ulaz[i]))
                {
                    sb.Append(ulaz[i]);
                }
            }
            dimenzijaProblema = Convert.ToInt32(sb.ToString());            
        }


        /// <summary>
        /// Parsira se ulazni niz koji sadrzi min ili max a predstavlja funkciju cilja problema
        /// </summary>
        /// <param name="uvjet"></param>
        private void FunkcijaCilja(string uvjet)
        {
            uvjet = uvjet.Remove(0, 4);
            int i = 0;
            System.Globalization.NumberFormatInfo nfi = new System.Globalization.NumberFormatInfo();
            nfi.NumberGroupSeparator = ".";

            while (i < uvjet.Length)
            {
                StringBuilder sb = new StringBuilder();
                
                if (uvjet[i] == '-' || uvjet[i] == '+' || char.IsDigit(uvjet[i]))
                {
                    if(uvjet[i] == '-') sb.Append('-');

                    while (uvjet[i] != 'x' && uvjet[i] != '*')
                    {
                        if(char.IsDigit(uvjet[i]) || uvjet[i] == '.') sb.Append(uvjet[i]);
                        i++;
                    }

                    if ((sb.Length == 1 && sb.ToString().Contains("-")) || sb.Length == 0) sb.Append(1);

                    if (uvjet[i] == '*') i += 2;
                    else i++;

                    StringBuilder index = new StringBuilder();
                    while (i < uvjet.Length && char.IsDigit(uvjet[i]))
                    {
                        index.Append(uvjet[i]);
                        i++;
                    }
                    
                    matricaCilja[int.Parse(index.ToString()) - 1] = Convert.ToDouble(sb.ToString(), nfi);
                }

                if (i<uvjet.Length && uvjet[i] == 'x')
                {
                    i++;
                    StringBuilder index = new StringBuilder();
                    while (i < uvjet.Length && char.IsDigit(uvjet[i]))
                    {
                        index.Append(uvjet[i]);
                        i++;
                    }
                    matricaCilja[int.Parse(index.ToString()) - 1] = 1.0;
                }

                i++;
            }
        }


        /// <summary>
        /// Parsiraju se ulazni nizovi koji zapocinju ounakom u a koji predstavljaju uvjete zadanog problema
        /// </summary>
        /// <param name="uvjet">Uvjet problema</param>
        /// <param name="brojUvjeta">Pripadni broj uvjeta</param>
        private void Uvjeti(string uvjet, int brojUvjeta)
        {
            uvjet = uvjet.Remove(0, 2);
            int i = 0;
            System.Globalization.NumberFormatInfo nfi = new System.Globalization.NumberFormatInfo();
            nfi.NumberGroupSeparator = ".";

            while (i < uvjet.Length)
            {
                StringBuilder sb = new StringBuilder();

                if (uvjet[i] == '-' || uvjet[i] == '+' || char.IsDigit(uvjet[i]))
                {
                    if (uvjet[i] == '-') sb.Append('-');

                    while (uvjet[i] != 'x' && uvjet[i] != '*')
                    {
                        if (char.IsDigit(uvjet[i]) || uvjet[i] == '.') sb.Append(uvjet[i]);
                        i++;
                    }

                    if ((sb.Length == 1 && sb.ToString().Contains("-")) || sb.Length == 0) sb.Append(1);

                    if (uvjet[i] == '*') i += 2;
                    else i++;

                    StringBuilder index = new StringBuilder();
                    while (i < uvjet.Length && char.IsDigit(uvjet[i]))
                    {
                        index.Append(uvjet[i]);
                        i++;
                    }

                    matricaUvjeta[brojUvjeta, int.Parse(index.ToString()) - 1] = Convert.ToDouble(sb.ToString(), nfi);
                }

                else if (i < uvjet.Length && uvjet[i] == 'x')
                {
                    i++;
                    StringBuilder index = new StringBuilder();
                    while (i < uvjet.Length && char.IsDigit(uvjet[i]))
                    {
                        index.Append(uvjet[i]);
                        i++;
                    }
                    matricaUvjeta[brojUvjeta, int.Parse(index.ToString()) - 1] = 1.0;
                }

                else if (i < uvjet.Length && (uvjet[i] == '<' || uvjet[i] == '>' || uvjet[i] == '='))
                {
                    StringBuilder oper = new StringBuilder();
                    oper.Append(uvjet[i]);
                    if(uvjet[i+1] == '<' || uvjet[i+1] == '>' || uvjet[i+1] == '=')
                    {
                        i++;
                        oper.Append(uvjet[i]);
                    }

                    operatorOdnosa[brojUvjeta] = oper.ToString();

                    StringBuilder broj = new StringBuilder();
                    while (i < uvjet.Length)
                    {
                        if (char.IsDigit(uvjet[i]) || uvjet[i] == '.' || uvjet[i] == '-')
                        {
                            broj.Append(uvjet[i]);                            
                        }
                        i++;
                    }
                    matricaUvjeta[brojUvjeta, dimenzijaProblema] = double.Parse(broj.ToString(), nfi);
                }

                i++;
            }
        }

        /// <summary>
        /// Pokrece parsiranje ulaznog teksta
        /// </summary>
        public void Start()
        {
            textProblema = textProblema.Replace('\n',' ');            
            string[] temp = textProblema.Split(';');
            int brojUvjeta = 0;
            int i = 0;

            foreach (string uvjet in temp)
            {
                if (uvjet.Contains("u")) brojUvjeta++;
            }
            this.brojUvjeta = brojUvjeta;
            
            foreach (string uvjet in temp)
            {
                if (uvjet.Contains("dim"))
                {
                    Dimenzija(uvjet);
                    break;
                }
            }

            matricaCilja = new double[dimenzijaProblema];
            matricaUvjeta = new double[brojUvjeta, dimenzijaProblema + 1];
            operatorOdnosa = new string[brojUvjeta];
            foreach (string uvjet in temp)
            {                
                if (uvjet.Contains("max") || uvjet.Contains("min"))
                {
                    if (uvjet.Contains("max")) max = true;
                    else max = false;

                    FunkcijaCilja(uvjet);
                }

                else if (uvjet.Contains("u"))
                {
                    Uvjeti(uvjet, i++);
                }
                else if (uvjet.Contains("dim") || uvjet.Length == 0)
                {
                    continue;
                }
                else
                {
                    throw new FormatException("Neispravno zadan problem linearnog programa!");
                }
            }
        }       
    }
}
