using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.Serialization;

namespace LinearProgramming_SIMPLEX.GeneticAlgorithmClass
{
    [Serializable]
    class Problem : ISerializable
    {
        private double[] matricaCilja;
        private double[,] matricaUvjeta;
        private string[] operatorOdnosa;
        private int dimenzijaProblema;
        private int brojUvjeta;
        private double preciznostJednakoUvjeta;

        public Problem(double[] inMatricaCilja, double[,] inMatricaUvjeta, string[] inOperatorOdnosa, int inDimenzijaProblema, int inBrojUvjeta, double inPreciznostJednakoUvjeta)
        {
            matricaCilja = inMatricaCilja;
            matricaUvjeta = inMatricaUvjeta;
            operatorOdnosa = inOperatorOdnosa;
            dimenzijaProblema = inDimenzijaProblema;
            brojUvjeta = inBrojUvjeta;
            preciznostJednakoUvjeta = inPreciznostJednakoUvjeta;
        }

        public int DimenzijaProblema
        {
            get
            {
                return dimenzijaProblema;
            }
        }

        public double[] MatricaCilja
        {
            get
            {
                return matricaCilja;
            }
        }

        public double[,] MatricaUvjeta
        {
            get
            {
                return matricaUvjeta;
            }
        }

        public string[] MatricaOdnosa
        {
            get
            {
                return operatorOdnosa;
            }
        }

        public int BrojUvjeta
        {
            get
            {
                return brojUvjeta;
            }
        }

        public double PreciznostJednakoUvjeta
        {
            get
            {
                return preciznostJednakoUvjeta;
            }
        }

        public double IzracunajFunkCilja(Vektor v)
        {
            double rez = 0;

            for (int i = 0; i < dimenzijaProblema; i++)
            {
                rez += v[i] * matricaCilja[i];
            }
            return rez;
        }

        public bool Uvjeti(Vektor v)
        {
            int k = 0;
            bool z = false;

            for (int i = 0; i < brojUvjeta; i++)
            {
                double rez = 0;
                for (int j = 0; j < dimenzijaProblema; j++)
                {
                    rez += matricaUvjeta[i,j] * v[j];
                }

                switch (operatorOdnosa[k])
                {
                    case "<=":
                        if (rez <= matricaUvjeta[i,dimenzijaProblema]) z = true;
                        else z = false;
                        break;

                    case ">=":
                        if (rez >= matricaUvjeta[i,dimenzijaProblema]) z = true;
                        else z = false;
                        break;
                    case "=":
                        if (rez <= matricaUvjeta[i,dimenzijaProblema] + preciznostJednakoUvjeta && rez >= matricaUvjeta[i,dimenzijaProblema] - preciznostJednakoUvjeta) z = true;
                        else z = false;
                        break;
                    case "<":
                        if (rez < matricaUvjeta[i,dimenzijaProblema]) z = true;
                        else z = false;
                        break;
                    case ">":
                        if (rez > matricaUvjeta[i,dimenzijaProblema]) z = true;
                        else z = false;
                        break;
                }
                if (z == false) return false;
                k++;
            }


            return true;

        }

        #region ISerializable Members

        public Problem(SerializationInfo info, StreamingContext contex)
        {

            matricaCilja = info.GetValue("MatricaCilja", typeof(double[])) as double[];
            matricaUvjeta = info.GetValue("MatricaUvjeta", typeof(double[,])) as double[,];
            operatorOdnosa = info.GetValue("OperatorOdnosa", typeof(string[])) as string[];
            dimenzijaProblema = (int)info.GetValue("DimenzijaProblema", typeof(int));
            brojUvjeta = (int)info.GetValue("BrojUvjeta", typeof(int));
            preciznostJednakoUvjeta = (double)info.GetValue("PreciznostJednakoUvjeta", typeof(double));
        }

        public void GetObjectData(SerializationInfo info, StreamingContext context)
        {

            info.AddValue("MatricaCilja", matricaCilja);
            info.AddValue("MatricaUvjeta", matricaUvjeta);
            info.AddValue("OperatorOdnosa", operatorOdnosa);
            info.AddValue("DimenzijaProblema", dimenzijaProblema);
            info.AddValue("BrojUvjeta", brojUvjeta);
            info.AddValue("PreciznostJednakoUvjeta", preciznostJednakoUvjeta);
        }

        #endregion
    }
}
