using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;


namespace LinearProgramming_SIMPLEX.GeneticAlgorithmClass
{
    public partial class Simplex : Form
    {

        private StreamReader sr;
        private StreamWriter sw;
        private string trenutniFile;
        private int brojGeneracija;
        private int velicinaPopulacije;
        private double vjerojatnostMut;
        private int preciznostDecMjesta;
        private double preciznostIzracunaGranica;
        private double preciznostJednakoUvjeta;
        private System.Globalization.NumberFormatInfo nfi;

        private Parser parser = null;
        private Problem primjer = null;
        private GraniceProblema gp = null;
        private GenetskiAlgoritam ga = null;


        public Simplex()
        {
            InitializeComponent();            
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {            
            trenutniFile = null;
            problem.Clear();
            tablicaRezultata.Rows.Clear();
            tab.SelectTab(0);
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            openFile.FileName = "*.txt";
            if (openFile.ShowDialog() == DialogResult.OK)
            {                
                tab.SelectTab(0);
                problem.Clear();
                tablicaRezultata.Rows.Clear();

                trenutniFile = openFile.FileName;
                sr = new StreamReader(openFile.OpenFile());
                problem.Text = sr.ReadToEnd();
                sr.Close();
                tab.Show();
                putanja.Text = openFile.FileName;
                status.Clear();
            }
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (trenutniFile != null)
            {
                sw = new StreamWriter(trenutniFile);
                sw.Write(problem.Text);
                sw.Close();
            }
            else if (trenutniFile == null)
            {
                saveFile.FileName = "*.txt";
                if (saveFile.ShowDialog() == DialogResult.OK)
                {                    
                    sw = new StreamWriter(saveFile.FileName);
                    sw.Write(problem.Text);
                    trenutniFile = saveFile.FileName;
                    sw.Close();
                }
            }
        }

        private void saveAsToolStripMenuItem1_Click(object sender, EventArgs e)
        {
             saveFile.FileName = "*.txt";
             if (saveFile.ShowDialog() == DialogResult.OK)
             {
                 sw = new StreamWriter(saveFile.FileName);
                 sw.Write(problem.Text);
                 trenutniFile = saveFile.FileName;
                 sw.Close();
             }
        }
        private void exitToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            Application.Exit();
        } 

        private void InicijalizirajPodatkeGA()
        {
            velicinaPopulacije = int.Parse(brPopulacija.Text);
            brojGeneracija = int.Parse(brGeneracija.Text);
            nfi = new System.Globalization.NumberFormatInfo();
            nfi.NumberDecimalSeparator = ".";
            vjerojatnostMut = double.Parse(vjerojatnostMutacije.Text,System.Globalization.NumberStyles.Float, nfi);
            preciznostDecMjesta = int.Parse(preciznost.Text);
            preciznostIzracunaGranica = double.Parse(preciznostGranica.Text, System.Globalization.NumberStyles.Float, nfi);

            preciznostJednakoUvjeta = double.Parse(preciznostUvjeta.Text, System.Globalization.NumberStyles.Float, nfi);
            
        }

        private void IspisiRezultat(Vektor v)
        {
            string[] redak = new string[3];
            for (int i = 0; i < v.Velicina; i++)
            {
                redak[0] = "x" + (int)(i + 1);
                redak[1] = "[" + gp.DonjaGranica[i] + " : " + gp.GornjaGranica[i] + "]";
                redak[2] = Convert.ToString(v[i]);
                tablicaRezultata.Rows.Add(redak);
            }

            double rez = 0;
            for (int i = 0; i < primjer.DimenzijaProblema; i++)
            {
                rez += primjer.MatricaCilja[i] * v[i];
            }
            
            tablicaRezultata.Rows.Add();            
            redak[0] = "Funkcija cilja";
            redak[1] = "";
            redak[2] = Convert.ToString(rez);
            tablicaRezultata.Rows.Add(redak);
        }

        private void pokreniGA_Click(object sender, EventArgs e)
        {
            if (trenutniFile != null)
            {
                status.Clear();                
                InicijalizirajPodatkeGA();
                tablicaRezultata.Rows.Clear();

                try
                {
                    parser = new Parser(problem.Text);
                    parser.Start();
                    primjer = new Problem(parser.MatricaCilja, parser.MatricaUvjeta, parser.MatricaOdnosa, parser.DimenzijaProblema, parser.BrojUvjeta, preciznostJednakoUvjeta);
                    status.AppendText("Ispravno zadan problem linearnog programa! \nUspjeno generirane tablice!\n");
                }
                catch (FormatException fe)
                {
                    status.AppendText("Neispravno zadan problem!\n" + fe.Message);
                }


                try
                {
                    gp = new GraniceProblema(primjer, preciznostIzracunaGranica);
                    gp.IzracunajGranice();

                    proces.Maximum = 0;

                    GenetskiAlgoritam.BrojDecMjesta = preciznostDecMjesta;
                    GenetskiAlgoritam.BrojDimenzija = parser.DimenzijaProblema;

                    GenetskiAlgoritam.DonjaGranicaDimenzije = gp.DonjaGranica;
                    GenetskiAlgoritam.GornjaGranicaDimenzije = gp.GornjaGranica;

                    status.AppendText("Granice problema su odreene!\n");

                }
                catch (Exception ex)
                {
                    status.AppendText("Granice problema nisu odreene!\n" + ex.Message);
                }  

                ga = new GenetskiAlgoritam(velicinaPopulacije, vjerojatnostMut, brojGeneracija, parser.Max);

                proces.Maximum = 0;
                proces.Maximum = velicinaPopulacije + brojGeneracija;
                proces.Step = 1;

                if (ponovi.Checked == false)
                {

                    DateTime pocetak = DateTime.Now;                    

                    Vektor v = ga.PokreniGA(primjer, status, false);

                    DateTime kraj = DateTime.Now;

                    TimeSpan vrijeme = kraj - pocetak;

                    status.AppendText("Vrijeme obavljanja genetskog algoritma : " + vrijeme.TotalSeconds + "sekundi.\n");

                    proces.Maximum = 0;

                    IspisiRezultat(v);

                    tab.SelectTab(1);

                    status.ScrollToCaret();
                }
                else
                {
                    ArrayList poljeVektora = new ArrayList(int.Parse(brPonovi.Text));
                    int i = 0;

                    TimeSpan ukupno = new TimeSpan(0,0,0,0,0);                    
                    
                    while (i < int.Parse(brPonovi.Text))
                    {
                        proces.Maximum = velicinaPopulacije + brojGeneracija;

                        DateTime pocetak = DateTime.Now;
                        Vektor v = ga.PokreniGA(primjer, status, true);
                        DateTime kraj = DateTime.Now;

                        ukupno += (kraj - pocetak);

                        poljeVektora.Add(v);

                        proces.Maximum = 0;

                        status.AppendText("Broj obavljenih ponavljanja :: " + (int)(i+1) + "\n");

                        status.ScrollToCaret();                        

                        i++;
                    }
                    
                    double prosjekSek = ukupno.TotalSeconds / int.Parse(brPonovi.Text);

                    status.AppendText("...............Prosjeno vrijeme obavljanja genetskog algoritma :: " + prosjekSek + " sekundi. \n");
                    

                    Vektor suma = new Vektor(primjer.DimenzijaProblema);
                    for (i = 0; i < primjer.DimenzijaProblema; i++)
                    {
                        suma[i] = 0;
                    }
                    foreach (Vektor v in poljeVektora)
                    {
                        for (i = 0; i < primjer.DimenzijaProblema; i++)
                        {
                            suma[i] += v[i];
                        }
                    }

                    for (i = 0; i < primjer.DimenzijaProblema; i++)
                    {
                        suma[i] /= int.Parse(brPonovi.Text);
                    }

                    IspisiRezultat(suma);

                    tab.SelectTab(1);                    
                }
            }
            else
            {
                status.AppendText("Odredite ulaznu datoteku!\n");
            }
            
        }

        private void sintaksa_Click(object sender, EventArgs e)
        {
            try
            {
                if (trenutniFile != null)
                {
                    InicijalizirajPodatkeGA();
                    parser = new Parser(problem.Text);
                    parser.Start();
                    primjer = new Problem(parser.MatricaCilja, parser.MatricaUvjeta, parser.MatricaOdnosa, parser.DimenzijaProblema, parser.BrojUvjeta, preciznostJednakoUvjeta);
                    status.AppendText("Ispravno zadan problem linearnog programa! \nUspjeno generirane tablice!\n");
                }
                else
                {
                    status.AppendText("Datoteka nije uitanan!\n");
                }
            }
            catch (FormatException fe)
            {
                status.AppendText("Neispravno zadan problem!\n" + fe.Message);
            }            
        }

        private void granice_Click(object sender, EventArgs e)
        {            
            try
            {
                if (trenutniFile != null)
                {
                    InicijalizirajPodatkeGA();

                    if (primjer == null)
                    {
                        parser = new Parser(problem.Text);
                        parser.Start();
                        primjer = new Problem(parser.MatricaCilja, parser.MatricaUvjeta, parser.MatricaOdnosa, parser.DimenzijaProblema, parser.BrojUvjeta, preciznostJednakoUvjeta);
                    }
                    gp = new GraniceProblema(primjer, preciznostIzracunaGranica);
                    gp.IzracunajGranice();
                    status.AppendText("Granice problema su odreene!\n");
                    proces.Maximum = 0;
                }
                else
                {
                    status.AppendText("Datoteka nije uitana!\n");
                }
            }
            catch (Exception ex)
            {                
                status.AppendText("Granice problema nisu odreene!\n" + ex.Message);
            }            
        }

        private void ponovi_MouseClick(object sender, MouseEventArgs e)
        {
            if (ponovi.Checked == true)
            {
                brPonovi.ReadOnly = false;
            }
            else
            {
                brPonovi.ReadOnly = true;
            }
        }                    
    }
}