using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CommonClass;
using System.Runtime.Serialization;

namespace LinearProgramming_SIMPLEX.GeneticAlgorithmClass
{
    [Serializable]
    public partial class SimplexGAManagerSimple : CommonClass.AlgorithmManager, ISerializable
    {

        private Problem problem;
        private GraniceProblema graniceProblema;
        private GenetskiAlgoritam ga;

        public SimplexGAManagerSimple()
        {
            InitializeComponent();
        }

        private RichTextBox _status;
        public RichTextBox Status
        {
            get { return _status; }
            set { _status = value; }
        }
        public SimplexGAManagerSimple(RichTextBox status)
            :base()
        {
            InitializeComponent();
            _status = status;                        
        }

        public override void StartAlgorithm()
        {
            if (InputData.Count == 0)
                MessageBox.Show("Pokrenite parsiranje ulazne datoteke kako bi mogli pokrenuti algoritam!", "Upozorenje");
            else
            {
                Progress.Start(this.Location);
                MakeAlgorithm();
                Progress.End();

                FireFinishedAlgorithm();
            }
        }

        private void MakeProblem()
        {
            try
            {
                problem = new Problem(
                   (double[])_inputData[InputFileParsers.KeyWordsSimplex.matricaCilja],
                   (double[,])_inputData[InputFileParsers.KeyWordsSimplex.matricaUvjeta],
                   (string[])_inputData[InputFileParsers.KeyWordsSimplex.operatorOdnosa],
                   (int)_inputData[InputFileParsers.KeyWordsSimplex.dimenzijaProblema],
                   (int)_inputData[InputFileParsers.KeyWordsSimplex.brojUvjeta],
                   (double)nudPrecision.Value
                   );
                _status.AppendText("Ispravno zadan problem linearnog programa! \nUspjeno generirane tablice!\n");
            }
            catch (FormatException fe)
            {
                _status.AppendText("Neispravno zadan problem!\n" + fe.Message);
            }
        }

        private void SetGraniceProblema()
        {
            try
            {
                graniceProblema = new GraniceProblema(problem, dtbBorderPrecision.GetValue);
                graniceProblema.IzracunajGranice();
                _status.AppendText("Granice problema su odreene!\n");
            }
            catch (Exception ex)
            {
                _status.AppendText("Granice problema nisu odreene!\n" + ex.Message);
            }
        }

        private void MakeAlgorithm()
        {
            _status.Clear();
            List<Vektor> listVektor = new List<Vektor>();

            MakeProblem();
            SetGraniceProblema();

            GenetskiAlgoritam.BrojDecMjesta = (int)nudPrecision.Value;
            GenetskiAlgoritam.BrojDimenzija = (int)_inputData[InputFileParsers.KeyWordsSimplex.dimenzijaProblema];
            GenetskiAlgoritam.DonjaGranicaDimenzije = graniceProblema.DonjaGranica;
            GenetskiAlgoritam.GornjaGranicaDimenzije = graniceProblema.GornjaGranica;
            ga = new GenetskiAlgoritam(
                (int)nudPopulationNumber.Value,
                (double)dtbProbabilityMutation.GetValue,
                (int)nudGenerationNumber.Value,
                (bool)_inputData[InputFileParsers.KeyWordsSimplex.max]
                );

            if (!cbRepeat.Checked)
            {
                DateTime pocetak = DateTime.Now;

                listVektor.Add(ga.PokreniGA(problem, _status, false));

                DateTime kraj = DateTime.Now;

                TimeSpan vrijeme = kraj - pocetak;

                _status.AppendText("Vrijeme obavljanja genetskog algoritma : " + vrijeme.TotalSeconds + "sekundi.\n");

                _status.ScrollToCaret();
            }
            else
            {
                int numRepeat = (int)nudGARepeatNumber.Value;
                TimeSpan ukupno = new TimeSpan(0, 0, 0, 0, 0);
                int i = 0;
                while (i < numRepeat)
                {
                    DateTime pocetak = DateTime.Now;
                    Vektor v = ga.PokreniGA(problem, _status, true);
                    DateTime kraj = DateTime.Now;

                    ukupno += (kraj - pocetak);

                    listVektor.Add(v);

                    _status.AppendText("Broj obavljenih ponavljanja :: " + (int)(i + 1) + "\n");

                    _status.ScrollToCaret();

                    i++;
                }

                double prosjekSek = ukupno.TotalSeconds / numRepeat;

                _status.AppendText("...............Prosjeno vrijeme obavljanja genetskog algoritma :: " + prosjekSek + " sekundi. \n");
            }

            MakeResults(listVektor);

        }

        private void MakeResults(List<Vektor> list)
        {
            DataTable dataTable = new DataTable();
            OutputData.Clear();
            dataTable.Columns.Add("Dimenzija", typeof(string));
            dataTable.Columns.Add("Granica dimenzije",typeof(string));
            dataTable.Columns.Add("Rezultat", typeof(string));
            int k = 0;
            foreach (Vektor v in list)
            {
                DataRow row;                
                if (cbRepeat.Checked)
                {
                    row = dataTable.NewRow();
                    row[0] = string.Format("{0}. ponavljanje:", k++ + 1);
                    row[1] = "";
                    row[2] = "";
                    dataTable.Rows.Add(row);
                }

                for (int i = 0; i < v.Velicina; i++)
                {
                    row = dataTable.NewRow();
                    row["Dimenzija"] = "x" + (int)(i + 1);
                    row["Granica dimenzije"] = "["
                        + graniceProblema.DonjaGranica[i]
                        + " : "
                        + graniceProblema.GornjaGranica[i] + "]";
                    row["Rezultat"] = Convert.ToString(v[i]);
                    dataTable.Rows.Add(row);
                }

                double rez = 0;
                for (int i = 0; i < v.Velicina; i++)
                    rez += problem.MatricaCilja[i] * v[i];
                row = dataTable.NewRow();
                row["Dimenzija"] = "z(x)";
                row["Granica dimenzije"] = "";
                row["Rezultat"] = Convert.ToString(rez);
                dataTable.Rows.Add(row);
            }

            if (cbRepeat.Checked)
            {
                //todo:: dodati prosjecnu vrijednost rezultatat!!!
            }

            OutputData.Add("Result", dataTable);

        }

        private void cbRepeat_CheckedChanged(object sender, EventArgs e)
        {
            nudGARepeatNumber.Enabled = cbRepeat.Checked;
        }

        #region ISerializable Members

        public SimplexGAManagerSimple(SerializationInfo info, StreamingContext contex)
            : base(info, contex)
        {

            InitializeComponent();

            problem = info.GetValue("Problem", typeof(Problem)) as Problem;
            graniceProblema = info.GetValue("GraniceProblema", typeof(GraniceProblema)) as GraniceProblema;
            ga = info.GetValue("GenetskiAlgoritam", typeof(GenetskiAlgoritam)) as GenetskiAlgoritam;
            _status = new RichTextBox();
            _status.Text = info.GetValue("Status", typeof(string)) as string;

            nudGARepeatNumber.Value = (int)info.GetValue("nudGARepeatNumber", typeof(int));
            nudGenerationNumber.Value = (int)info.GetValue("nudGenerationNumber", typeof(int));
            nudPopulationNumber.Value = (int)info.GetValue("nudPopulationNumber", typeof(int));
            nudPrecision.Value = (int)info.GetValue("nudPrecision", typeof(int));
            dtbBorderPrecision.Text = info.GetValue("dtbBorderPrecision", typeof(string)) as string;
            dtbProbabilityMutation.Text = info.GetValue("dtbProbabilityMutation", typeof(string)) as string;
            dtpEqualPrecision.Text = info.GetValue("dtpEqualPrecision", typeof(string)) as string;
            cbRepeat.Checked = (bool)info.GetValue("cbRepeat", typeof(bool));
        }

        void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
        {
            base.GetObjectData(info, context);

            info.AddValue("Problem", problem);
            info.AddValue("GraniceProblema", graniceProblema);
            info.AddValue("GenetskiAlgoritam", ga);
            info.AddValue("Status", _status.Text);

            info.AddValue("nudGARepeatNumber", nudGARepeatNumber.Value);
            info.AddValue("nudGenerationNumber", nudGenerationNumber.Value);
            info.AddValue("nudPopulationNumber", nudPopulationNumber.Value);
            info.AddValue("nudPrecision", nudPrecision.Value);
            info.AddValue("dtbBorderPrecision", dtbBorderPrecision.Text);
            info.AddValue("dtbProbabilityMutation", dtbProbabilityMutation.Text);
            info.AddValue("dtpEqualPrecision", dtpEqualPrecision.Text);
            info.AddValue("cbRepeat", cbRepeat.Checked);
        }

        #endregion
    }
}

