using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CommonClass;
using System.Reflection;
using System.Runtime.Serialization;
using LinearProgramming_SIMPLEX.GeneticAlgorithmClass;


namespace LinearProgramming_SIMPLEX
{
    [Serializable]
    public partial class LinearProgrammingProblem : CommonClass.WorkingSession, ISerializable
    {

        protected override void InitializeComponentAndData()
        {
            InitializeComponent();
        }

        public LinearProgrammingProblem()
        {
            InitializeComponentAndData();
        }
        public LinearProgrammingProblem(Dictionary<string, object> extendData, ImplementedAlgorithmType algorithmType)
            : base(extendData) 
        {            
            switch (algorithmType)
            {
                case ImplementedAlgorithmType.GeneticAlgorithm:
                    algorithmManager = new GeneticAlgorithmClass.SimplexGAManagerSimple(rtbOutput);
                    //algorithmManager.FinishedAlgorithm += new FinishedAlgorithmHandler(algorithmManager_FinishedAlgorithm);
                    break;
                case ImplementedAlgorithmType.SI:
                    break;
                case ImplementedAlgorithmType.error:
                    throw new Exception("Nepostojei algoritam!");
            }
            RegisterFinishedAlgorithmEvent();
            ShowAlgorithmManager();
        }

        protected override void RegisterFinishedAlgorithmEvent()
        {
            algorithmManager.FinishedAlgorithm += new FinishedAlgorithmHandler(algorithmManager_FinishedAlgorithm);
        }

        void algorithmManager_FinishedAlgorithm()
        {
            tablicaRezultata.DataSource = (algorithmManager.OutputData["Result"] as DataTable);

            tabResulats.SelectTab(1);
        }

        protected override DataTable SetInputFileTypes(DataTable data, string description, string name)
        {
            DataRow row = data.NewRow();

            row[description] = "SIMPLEX";
            row[name] = "simplex";

            data.Rows.Add(row);

            return data;
        }

        protected override DataTable SetOutputFileTypes(DataTable data, string description, string name)
        {
            DataRow row = data.NewRow();

            row[description] = "WriteResults";
            row[name] = "WriteResults";

            data.Rows.Add(row);

            return data;
        }

        protected override string SetupInputFileExtension()
        {
            return "SIMPLEX (*.simplex) |*.simplex";
        }

        protected override CommonClass.IInputParser ProblemDefinition(string inputText, string selectedExtension)
        {
            rtbProblem.Text = inputText;
            rtbProblem.BringToFront();
            return DynamicalCall.GetInputParser(selectedExtension, inputText, Assembly.GetExecutingAssembly());
        }

        protected override IOutputParser SetOutputParser(string outputParserType)
        {
            return DynamicalCall.GetOutputParser(outputParserType, Assembly.GetExecutingAssembly());
        }

        protected override IInputParser SetInputParser(string inputParserType)
        {
            return DynamicalCall.GetInputParser(inputParserType, "", Assembly.GetExecutingAssembly());
        }

        protected override string SetInputText()
        {
            return rtbProblem.Text;
        }

        protected override string SetupOuputFileExtension()
        {
            return "Results | *.out";
        }

        protected override void MakeNewProblem()
        {
            this.rtbProblem.Clear();
            this.rtbProblem.BringToFront();
        }

        protected override string ProblemToString()
        {
            return rtbProblem.Text;
        }

        private void rtbProblem_TextChanged(object sender, EventArgs e)
        {
            IsInputFileModify = true;
        }        

        #region ISerializable Members

        public LinearProgrammingProblem(SerializationInfo info, StreamingContext context)
            :base(info, context)
        {
            this.rtbProblem.Text = (string)info.GetValue("problemDefinition", typeof(string));
            this.tablicaRezultata.DataSource = info.GetValue("tablicaRezultata", typeof(DataTable)) as DataTable;

            RegisterFinishedAlgorithmEvent();
            ShowAlgorithmManager();

            if (algorithmManager is SimplexGAManagerSimple)
                (algorithmManager as SimplexGAManagerSimple).Status = rtbOutput;
        }

        void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
        {
            GetObjectData(info, context);
            info.AddValue("problemDefinition", rtbProblem.Text);
            info.AddValue("tablicaRezultata", tablicaRezultata.DataSource);
            
        }

        #endregion
    }
}

