using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.Odbc;
using System.Windows.Forms;

namespace Optimization_Framework.DataBaseConnection
{
    public class DataBaseEntry
    {
        /// <summary>
        /// Konekcijski string na bazu podataka DataBase.mdb, koja se nalazi gdje i exe programa
        /// </summary>
        private static readonly string _connectionString = @"Driver={Microsoft Access Driver (*.mdb)};Dbq=" + Environment.CurrentDirectory + @"\DataBase.mdb;Uid=;Pwd=;";

        public DataBaseEntry() { }

        private DataTable ExecuteCommand(string sqlCommand)
        {
            DataTable data = new DataTable();
            OdbcConnection conn = new OdbcConnection(_connectionString);
            try
            {
                conn.Open();
                OdbcCommand command = new OdbcCommand(sqlCommand, conn);
                OdbcDataAdapter adapter = new OdbcDataAdapter(command);
                adapter.Fill(data);
                return data;
            }
            catch (OdbcException e)
            {
                MessageBox.Show(e.Message, "Greka", MessageBoxButtons.OK);
            }
            finally
            {
                conn.Close();
            }

            return data;
        }

        public DataSet GetProblemTypesAndAllProblems()
        {
            DataTable problemTypes = ExecuteCommand("SELECT * FROM ProblemType");
            problemTypes.TableName = "problemTypes";

            DataTable problems = ExecuteCommand("SELECT * FROM Problems");
            problems.TableName = "problems";

            DataSet data = new DataSet();
            data.Tables.AddRange(new DataTable[] { problemTypes, problems });

            data.Relations.Add("relation_problemTypes_problems", problemTypes.Columns["ID"], problems.Columns["IDProblemType"]);

            return data;

        }

        public DataSet GetAlgorithmTypesAndAlgorithms(int idProblem)
        {
            DataTable algorihmTypes = ExecuteCommand("SELECT * FROM AlgorithmType");
            algorihmTypes.TableName = "algorihmTypes";

            DataTable algorithms = ExecuteCommand(
                string.Format(@"SELECT Algorithms.*, Relation.ComplitedAlgorithmName FROM Algorithms,  Relation WHERE IDProblem = {0} AND IDAlgorithm = Algorithms.ID", idProblem)
                );
            algorithms.TableName = "algorithms";

            DataSet data = new DataSet();
            data.Tables.AddRange(new DataTable[] { algorihmTypes, algorithms });

            data.Relations.Add("relation_algorihmTypes_algorithms", algorihmTypes.Columns["ID"], algorithms.Columns["IDAlgorithmType"]);

            return data;
        }

    }
}
