using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Optimization_Framework.Wizard_control;
using Optimization_Framework.DataBaseConnection;
using System.Threading;
using CommonClass;

namespace Optimization_Framework
{
    public partial class Wizard : Form
    {

        /// <summary>
        /// Spajanje na bazu
        /// </summary>
        protected DataBaseEntry _dataEntry = new DataBaseEntry();

        public Wizard()
        {
            InitializeComponent();
            FillProblemsView();
        }

        private void FillProblemsView()
        {
            tvProblems.SelectedImageIndex = -1;
            tvProblems.ImageIndex = -1;
            DataSet data = _dataEntry.GetProblemTypesAndAllProblems();

            foreach (DataRow row in data.Tables["ProblemTypes"].Rows)
            {
                TreeNodeExtend rootNode = new TreeNodeExtend((string)row["Name"]);
                rootNode.ExtendPropertis.Add("ID", row["ID"]);
                if (row["Icon"] != DBNull.Value)
                {
                    System.IO.MemoryStream ms = new System.IO.MemoryStream((byte[])row["Icon"], true);
                    ms.Write((byte[])row["Icon"], 78, ((byte[])row["Icon"]).Length - 78);
                    Bitmap bitmap = new Bitmap(ms);
                    imageListProblems.Images.Add(string.Format("{0}_rootImage", Convert.ToString((int)row["ID"])), bitmap);
                    rootNode.ImageIndex = rootNode.SelectedImageIndex = rootNode.StateImageIndex = imageListProblems.Images.IndexOfKey(string.Format("{0}_rootImage", Convert.ToString((int)row["ID"])));
                }

                foreach (DataRow chiledRow in row.GetChildRows(data.Relations["relation_problemTypes_problems"]))
                {
                    TreeNodeExtend chiledNode = new TreeNodeExtend((string)chiledRow["Name"]);
                    chiledNode.ExtendPropertis.Add("ID", chiledRow["ID"]);

                    if (chiledRow["Icon"] != DBNull.Value)
                    {
                        System.IO.MemoryStream ms = new System.IO.MemoryStream((byte[])chiledRow["Icon"], true);
                        ms.Write((byte[])chiledRow["Icon"], 78, ((byte[])chiledRow["Icon"]).Length - 78);
                        Bitmap bitmap = new Bitmap(ms);
                        imageListProblems.Images.Add(Convert.ToString(chiledRow["ID"]), bitmap);
                        chiledNode.ImageIndex = chiledNode.SelectedImageIndex = chiledNode.StateImageIndex = imageListProblems.Images.IndexOfKey(Convert.ToString(chiledRow["ID"]));
                    }

                    rootNode.Nodes.Add(chiledNode);
                }
                tvProblems.Nodes.Add(rootNode);
            }

            tvProblems.ExpandAll();
        }

        private void FillAlgorithmsView(int idProblem)
        {
            lvAlgorithms.Clear();
            DataSet data = _dataEntry.GetAlgorithmTypesAndAlgorithms(idProblem);

            foreach (DataRow row in data.Tables["algorihmTypes"].Rows)
            {
                ListViewGroup group = new ListViewGroup((string)row["Name"]);
                group.Header = (string)row["Name"];
                group.HeaderAlignment = HorizontalAlignment.Left;
                lvAlgorithms.Groups.Add(group);
                foreach (DataRow chiledRow in row.GetChildRows(data.Relations["relation_algorihmTypes_algorithms"]))
                {
                    ListViewItemExtend item = new ListViewItemExtend((string)chiledRow["Name"]);
                    item.ExtendPropertis.Add("ID", chiledRow["ID"]);
                    item.ExtendPropertis.Add("SaveFileExtension", chiledRow["SaveFileExtension"]);
                    item.ExtendPropertis.Add("ComplitedAlgorithmName", chiledRow["ComplitedAlgorithmName"]);

                    if (chiledRow["Icon"] != DBNull.Value)
                    {
                        System.IO.MemoryStream ms = new System.IO.MemoryStream((byte[])chiledRow["Icon"], true);
                        ms.Write((byte[])chiledRow["Icon"], 78, ((byte[])chiledRow["Icon"]).Length - 78);
                        Bitmap bitmap = new Bitmap(ms);
                        imageListAlgorithms.Images.Add(Convert.ToString(chiledRow["ID"]), bitmap);
                        item.ImageIndex = imageListAlgorithms.Images.IndexOfKey(Convert.ToString(chiledRow["ID"]));
                    }
                    item.Group = group;
                    lvAlgorithms.Items.Add(item);
                }
            }
        }

        private void tvProblems_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (tvProblems.SelectedNode != null)
                FillAlgorithmsView((int)(tvProblems.SelectedNode as TreeNodeExtend).ExtendPropertis["ID"]);
        }

        private void lvAlgorithms_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lvAlgorithms.SelectedItems.Count != 0)
            {
                pProjectSave.Enabled = true;
                btnLoad.Enabled = true;
            }
            else
            {
                pProjectSave.Enabled = false;
                btnLoad.Enabled = false;
            }
        }

        /// <summary>
        /// Zbog spremanja i lodanja projekata
        /// </summary>
        /// <returns>Vraca Ekstenziju datoteke</returns>
        private string GetFileExtension()
        {
            return (lvAlgorithms.SelectedItems[0] as ListViewItemExtend).ExtendPropertis["SaveFileExtension"] != DBNull.Value ? (string)(lvAlgorithms.SelectedItems[0] as ListViewItemExtend).ExtendPropertis["SaveFileExtension"] : "of";
        }

        private void btnBrowse_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog fbd = new FolderBrowserDialog();
            if (fbd.ShowDialog() == DialogResult.OK)
            {
                tbProjectPath.Text = string.Format(@"{0}\{1}.{2}", fbd.SelectedPath, tbProjectName.Text, GetFileExtension());
                btnFinish.Enabled = true;
            }
            else
                btnFinish.Enabled = false;
        }

        private void btnFinish_Click(object sender, EventArgs e)
        {
            SelectedProblem();
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Dodati novi algoritam u novi case
        /// </summary>
        /// <returns></returns>
        ImplementedAlgorithmType GetTypeOfSelectedAlgorithm()
        {
            switch ((lvAlgorithms.SelectedItems[0] as Optimization_Framework.Wizard_control.ListViewItemExtend).Text)
            {
                case "Genetic Algorithm":
                    return ImplementedAlgorithmType.GeneticAlgorithm;
                case "SI":
                    return ImplementedAlgorithmType.SI;
                default:
                    MessageBox.Show("Odabrani algoritam nije implementiran!", "Greka");
                    return ImplementedAlgorithmType.error;
            }
        }

        /// <summary>
        /// Dodati novi problem u novi case
        /// </summary>
        private WorkingSession session = null;
        private void SelectedProblem()
        {
            if (lvAlgorithms.SelectedItems.Count == 0)
            {
                MessageBox.Show("Odaberite algoritam!", "Greka");
            }
            Dictionary<string, object> extendData = (lvAlgorithms.SelectedItems[0] as ListViewItemExtend).ExtendPropertis;
            extendData.Add("saveProject", tbProjectPath.Text);

            switch ((ComplitedProjects)Enum.Parse(typeof(ComplitedProjects), (lvAlgorithms.SelectedItems[0] as ListViewItemExtend).ExtendPropertis["ComplitedAlgorithmName"].ToString(), true))
            {
                case ComplitedProjects.simplexGA:
                    session = new LinearProgramming_SIMPLEX.LinearProgrammingProblem(extendData, GetTypeOfSelectedAlgorithm());
                    break;
                case ComplitedProjects.tspGA:
                    session = new TravelingSalesmanProblem_TSP.TravelingSalesmanProblem_TSP(extendData, GetTypeOfSelectedAlgorithm());
                    break;
                case ComplitedProjects.CapitalBudgetingProblem:
                    session = new CapitalBudgetingProblem.CapitalBudgetingProblem(extendData, GetTypeOfSelectedAlgorithm());
                    break;
                case ComplitedProjects.QuadraticAssignmentProblem:
                    session = new QuadraticAssignmentProblem.QuadraticAssignmentProblem(extendData, GetTypeOfSelectedAlgorithm());
                    break;
                case ComplitedProjects.P_MedianProblem:
                    session = new P_MedianProblem.P_MedianProblem(extendData, GetTypeOfSelectedAlgorithm());
                    break;
                default:
                    MessageBox.Show("Odabrani projekt nije implementiran!", "Greka");
                    return;
            }
            if (session != null)
            {
                session.MdiParent = this.MdiParent;
                session.Show();                
            }

            this.Close();
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            if(lvAlgorithms.SelectedItems.Count == 0)
            {
                MessageBox.Show("Odaberi problem i algoritam!", "Upozornje");
                return;
            }
            WorkingSession session = null;
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = string.Format("Optimization Framework|*.{0}", (lvAlgorithms.SelectedItems[0] as ListViewItemExtend).ExtendPropertis["SaveFileExtension"] as String);

            if (ofd.ShowDialog() == DialogResult.OK)
            {
                session = ProjectSerializer.LoadProject(ofd.FileName);
                session.MdiParent = this.MdiParent;
                session.Show();
                this.Close();
            }
        }
    }
}