using System;
using System.Collections.Generic;
using System.Text;
using CommonClass;
using System.IO;

namespace P_MedianProblem.InputFileParsers
{
    public enum KeyWordsPMP
    {
        vertices,
        edges,
        facilities,
        relationshipVertices
    };

    [Serializable]
    public class Graphs : IInputParser
    {
        private int _vertices;
        private int _edges;
        private int _facilities;
        private double[,] _relationshipVertices;

        private string _inputText;

        public Graphs(string inputText)
        {
            _inputText = inputText;
        }

        public Graphs(StreamReader inputFile)
        {
            _inputText = inputFile.ReadToEnd();
        }

        #region IParser Members

        public Dictionary<object, object> GetResult
        {
            get 
            {
                Dictionary<object, object> result = new Dictionary<object, object>();
                result.Add(KeyWordsPMP.vertices, _vertices);
                result.Add(KeyWordsPMP.edges, _edges);
                result.Add(KeyWordsPMP.facilities, _facilities);
                result.Add(KeyWordsPMP.relationshipVertices, _relationshipVertices);

                return result;
            }
        }

        public void Start()
        {
            string[] lines = _inputText.Split(new string[] { Environment.NewLine }, StringSplitOptions.RemoveEmptyEntries);
            if (lines.Length == 1)
            {
                lines = _inputText.Split(new string[] { "\n" }, StringSplitOptions.RemoveEmptyEntries);
            }

            foreach (string line in lines)
            {
                if (line.StartsWith("p"))
                    SetInformation(line);
                else if (line.StartsWith("e"))
                    SetRelationships(line);
                else
                    throw new ParserException("Nepostojeci format datoteke!");
            }
        }

        int numRelations = 0;
        private void SetRelationships(string line)
        {
            string[] data = line.Split(' ');
            try
            {
                _relationshipVertices[numRelations, 0] = double.Parse(data[1]);
                _relationshipVertices[numRelations, 1] = double.Parse(data[2]);
                _relationshipVertices[numRelations, 2] = double.Parse(data[3]);

                numRelations++;
            }
            catch
            {
                throw new ParserException("Krivo zadani podaci u odnosima!");
            }
        }

        private void SetInformation(string line)
        {
            string[] data = line.Split(' ');
            try
            {
                _vertices = int.Parse(data[1]);
                _edges = int.Parse(data[2]);
                _facilities = int.Parse(data[3]);

                _relationshipVertices = new double[_edges, 3];
            }
            catch 
            {
                throw new ParserException("Krivo zadani podaci o P-Median problemu!");
            }
        }

        #endregion

        #region IInputParser Members
        public string SetInputText
        {
            set { _inputText = value; }
        }

        #endregion
    }
}
