using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CommonClass;
using System.Runtime.Serialization;
using System.Reflection;

namespace P_MedianProblem
{
    [Serializable]
    public partial class P_MedianProblem : WorkingSession, ISerializable
    {
        protected override void InitializeComponentAndData()
        {
            InitializeComponent();
        }

        public P_MedianProblem()
        {
            InitializeComponentAndData();
        }

        //public P_MedianProblem(Dictionary<string, object> extendData, IAlgorithm algorithm)
        public P_MedianProblem(Dictionary<string, object> extendData, ImplementedAlgorithmType algorithmType)
            : base(extendData) 
        {
            switch (algorithmType)
            {
                case ImplementedAlgorithmType.GeneticAlgorithm:
                    algorithmManager = new AlgorithmManager();
                    break;
                case ImplementedAlgorithmType.SI:
                    break;
                case ImplementedAlgorithmType.error:
                    throw new Exception("Nepostojei algoritam!");
            }

            RegisterFinishedAlgorithmEvent();
            ShowAlgorithmManager();  
        }       

        protected override string SetupInputFileExtension()
        {
            return "Graph Form (*.graphs) |*.graphs";            
        }

        protected override IInputParser ProblemDefinition(string inputText, string selectedExtension)
        {
            rtbProblem.Text = inputText;
            rtbProblem.BringToFront();
            return DynamicalCall.GetInputParser(selectedExtension, inputText, Assembly.GetExecutingAssembly());
        }

        protected override string SetInputText()
        {
            return rtbProblem.Text;
        }

        protected override string SetupOuputFileExtension()
        {
            return "Results | *.out";
        }

        protected override DataTable SetInputFileTypes(DataTable data, string description, string name)
        {
            DataRow row = data.NewRow();
            row[description] = "Graph Form";
            row[name] = "graphs";

            data.Rows.Add(row);

            return data;
        }
        protected override DataTable SetOutputFileTypes(DataTable data, string description, string name)
        {
            DataRow row = data.NewRow();
            row[description] = "Results";
            row[name] = "out";

            data.Rows.Add(row);

            return data;
        }

        protected override IInputParser SetInputParser(string inputParserType)
        {
            return DynamicalCall.GetInputParser(inputParserType, "", Assembly.GetExecutingAssembly());
        }
        


        #region ISerializable Members

        public P_MedianProblem(SerializationInfo info, StreamingContext context)
            :base(info, context)
        {
            this.rtbProblem.Text = (string)info.GetString("problemDefinition");

            RegisterFinishedAlgorithmEvent();
        }

        void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
        {
            GetObjectData(info, context);
            info.AddValue("problemDefinition", rtbProblem.Text);
        }

        #endregion
    }
}

