using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using CommonClass;

namespace QuadraticAssignmentProblem.InputFileParsers
{
    public enum KeyWordsQAP
    {
        mA,
        mB,
        mC
    };

    /// <summary>
    /// Koopmans Beckmann Form
    /// </summary>
    [Serializable]
    public class Kbf : IInputParser
    {
        private double[,] _matrixA;
        private double[,] _matrixB;
        private double[,] _matrixC;
        private int _dimension;


        public Dictionary<object, object> GetResult
        {
            get
            {
                Dictionary<object, object> result = new Dictionary<object, object>();
                result.Add(KeyWordsQAP.mA, _matrixA);
                result.Add(KeyWordsQAP.mB, _matrixB);
                result.Add(KeyWordsQAP.mC, _matrixC);

                return result;
            }
        }
       
        private string _inputText;

        #region IInputParser Members
        public string SetInputText
        {
            set { _inputText = value; }
        }

        #endregion

        public Kbf(string inputText)
        {
            _inputText = inputText;
        }

        public Kbf(StreamReader inputFile)
        {
            _inputText = inputFile.ReadToEnd();
        }

        public Kbf() { }


        private void SetDimension(string dim)
        {
            int.TryParse(dim, out _dimension);
        }


        private void SetMatrixA(string a)
        {
            _matrixA = new double[_dimension, _dimension];

            a = a.Remove(0,1);            
            string[] temp = a.Split(new string[] { Environment.NewLine }, StringSplitOptions.RemoveEmptyEntries);
            if (temp.Length == 1)
            {
                a = a.Replace("\n", Environment.NewLine);
                temp = a.Split(new string[] { Environment.NewLine }, StringSplitOptions.RemoveEmptyEntries);
            }

            for (int i = 0; i < _dimension;i++)
            {
                string[] elements = temp[i].Split(' ');
                for(int j=0;j<_dimension;j++)
                {
                    double.TryParse(elements[j], out _matrixA[i, j]);
                }
            }
        }
        private void SetMatrixB(string b)
        {
            _matrixB = new double[_dimension, _dimension];
            b = b.Remove(0,1);
            string[] temp = b.Split(new string[] { Environment.NewLine }, StringSplitOptions.RemoveEmptyEntries);
            if (temp.Length == 1)
            {
                b = b.Replace("\n", Environment.NewLine);
                temp = b.Split(new string[] { Environment.NewLine }, StringSplitOptions.RemoveEmptyEntries);
            }

            for (int i = 0; i < _dimension;i++)
            {
                string[] elements = temp[i].Split(' ');
                for(int j=0;j<_dimension;j++)
                {
                    double.TryParse(elements[j], out _matrixB[i, j]);
                }
            }
        }
        private void SetMatrixC(string c)
        {
            _matrixC = new double[_dimension, _dimension];
            c = c.Remove(0, 1);
            string[] temp = c.Split(new string[] { Environment.NewLine }, StringSplitOptions.RemoveEmptyEntries);
            if (temp.Length == 1)
            {
                c = c.Replace("\n", Environment.NewLine);
                temp = c.Split(new string[] { Environment.NewLine }, StringSplitOptions.RemoveEmptyEntries);
            }

            for (int i = 0; i < _dimension; i++)
            {
                string[] elements = temp[i].Split(' ');
                for (int j = 0; j < _dimension; j++)
                {
                    double.TryParse(elements[j], out _matrixC[i, j]);
                }
            }
        }

        public void Start()
        {
            string[] temp = _inputText.Split(new char[] { 'm' }, 4, StringSplitOptions.None);

            SetDimension(temp[0]);

            foreach (string s in temp)
            {
                switch (s[0])
                {
                    case 'A':
                        SetMatrixA(s);
                        break;
                    case 'B':
                        SetMatrixB(s);
                        break;
                    case 'C':
                        SetMatrixC(s);
                        break;
                    default:
                        break;
                }
            }

        }



      
    }
}

