using System;
using System.Collections;
using System.Runtime.Serialization;

namespace TravelingSalesmanProblem_TSP.GeneticAlgorithmClass
{
	/// <summary>
	/// Kompletna logika genetskog algoritma.
	/// </summary>
    [Serializable]
	public class Algoritam : ISerializable
	{
		private double[][] udaljenosti;
		private int _brojGradova,_velPopulacije;
		private int _vjerSelekcije;
		private double _epsilon;
		public ArrayList Populacija;
		public ArrayList SortPop;
		
		/// <summary>
		/// Konstruktor koji inicijalizira algoritam za prirodnu selekciju.
		/// </summary>
		/// 
		public Algoritam(int brojClanova,int brGradova,int vjerSel,double epsilon,double[][] matricaUdalj)
		{
			udaljenosti = new double[brGradova][];

			for (int i = 0; i < brGradova;i++)
			{
				udaljenosti[i] = new double[brGradova];
				for (int j = 0; j < brGradova; j++)
				{
					udaljenosti[i][j] = matricaUdalj[i][j];
				}
			}
			this.Populacija = new ArrayList();
			this.SortPop  = new ArrayList();
			this._brojGradova = brGradova;
			this._vjerSelekcije = vjerSel;
			this._epsilon = epsilon;
			this._velPopulacije = brojClanova;
			inicijaliziraj(brojClanova);
		}
		
		/// <summary>
		/// Konstruktor koji inicijalizira algoritam za turnirsku selekciju
		/// </summary>
		public Algoritam(int brojClanova,int brGradova,double[][] matricaUdalj)
		{
			udaljenosti = new double[brGradova][];

			for (int i = 0; i < brGradova;i++)
			{
				udaljenosti[i] = new double[brGradova];
				for (int j = 0; j < brGradova; j++)
				{
					udaljenosti[i][j] = matricaUdalj[i][j];
				}
			}
			this.Populacija = new ArrayList();
			this.SortPop  = new ArrayList();
			this._brojGradova = brGradova;
			this._velPopulacije = brojClanova;

			inicijaliziraj(brojClanova);
		}
	
		/// <summary>
		/// Metoda koja stvara poetnu populaciju.
		/// </summary>
		private void inicijaliziraj(int brojClanova)
		{
			ArrayList elementi = new ArrayList();
			Random broj = new Random();
			for(int i = 0; i < brojClanova;i++)
			{
				for(int j = 0;j<_brojGradova;j++)
				{
					int rndBroj = broj.Next(_brojGradova);	
					if(elementi.Contains(rndBroj))
					{
						j--;
					}
					else
					{
						elementi.Add(rndBroj);
				
					}
				}
				//generiranje novog kromosoma iz stvorene staze
				Kromosom jedinka = new Kromosom(elementi,this.izracunajDobrotu(elementi));
				Populacija.Add(jedinka);
				elementi.Clear();
			}
		}
		/// <summary>
		/// Metoda koja rauna dobrotu pojedinog kromosoma. Dobrota je izraena kao duljina puta.
		/// </summary>
		private double izracunajDobrotu(ArrayList kromosom)
		{
			double dobrota = 0;
			
			foreach(int i in kromosom)
			{
				int next = (kromosom.IndexOf(i)+1) % _brojGradova;
				dobrota += udaljenosti[i][(int)kromosom[next]];
			}
			return dobrota;
		}
		/// <summary>
		/// sortirenj za prirodnu selekciju
		/// </summary>
		private void sort()
		{
			foreach(Kromosom k in this.Populacija)
				this.SortPop.Add(k);
			SortPop.Sort();
			Populacija.Sort();
			
		}

		/// <summary>
		/// Eliminacijska selekcija. Radi tako da eliminira kromosome s slinom vrijednou duljine puta,
		/// a nakon njih one s najloijom vrijednosu dobrote dok ne dodje do broja 
		/// jedinki.odreenog za eliminaciju.
		/// </summary>
		public void PrirodnaSelekcija ()
		{
			int R;
			R = System.Convert.ToInt32( this._velPopulacije * this._vjerSelekcije / 100);
			bool go = false;
			ArrayList zaMaknuti = new ArrayList();
			this.sort();
			//eliminiranje slicnih, tj razlika manja od epsilon
			foreach(Kromosom k in SortPop)
			{
				if (!go)
				{
					go = true;
				}
				else
				{
					int pred = SortPop.IndexOf(k)-1;
					if(((((Kromosom)SortPop[pred]).DuljinaPuta) - k.DuljinaPuta) < this._epsilon)
					{
						R--;
						zaMaknuti.Add(pred);
					}
					if (R == 0)
						break;
				}
			}
				
			for (int i = 0; i < zaMaknuti.Count;i++)
			{
				this.SortPop.RemoveAt((int)zaMaknuti[i]);
				
				for(int j=i+1;j<zaMaknuti.Count;j++)
				{
					int temp = (int)zaMaknuti[j];
					temp--;
					zaMaknuti[j]=temp;
				}
			
			}
			//micanje najlosijih dok se ne dodje do postotka koji treba biti eliminiran
			if (R > 0)
			{
				while (R > 0)
				{
					SortPop.RemoveAt(0);
					R--;
				}
			}
		}
		/// <summary>
		/// Metoda koja obavlja proces greedy crossover-a
		/// </summary>
		public Kromosom GreedyCrossover (Kromosom Rod1, Kromosom Rod2)
		{	
			ArrayList elementi = new ArrayList();
			
			ArrayList stazaRod1 = new ArrayList();
			ArrayList stazaRod2 = new ArrayList();

			stazaRod1 = Rod1.Chromosome;
			stazaRod2 = Rod2.Chromosome;
			elementi.Add(stazaRod1[0]);
			Random rnd = new Random();

			for (int i =1; i< stazaRod1.Count;i++)
			{

				int ind1 = (stazaRod1.IndexOf(elementi[i-1])+1) % this._brojGradova;
				int ind2 = (stazaRod2.IndexOf(elementi[i-1])+1) % this._brojGradova;
				int ind0 = (int)elementi[i-1];
				
				if((elementi.IndexOf(stazaRod1[ind1]) != -1) && (elementi.IndexOf(stazaRod2[ind2]) != -1))
				{
					int novi;
					do
					{
						novi = rnd.Next(this._brojGradova);
					}while(elementi.IndexOf(novi) != -1);
					elementi.Add(novi);
					continue;
				}
				if(elementi.IndexOf(stazaRod1[ind1]) != -1)
				{
					elementi.Add(stazaRod2[ind2]);
					continue;
				}
				if(elementi.IndexOf(stazaRod2[ind2]) != -1)
				{
					elementi.Add(stazaRod1[ind1]);
					continue;
				}
				
				if(udaljenosti[ind0][(int)stazaRod1[ind1]] <= udaljenosti[ind0][(int)stazaRod2[ind2]])
				{
					elementi.Add(stazaRod1[ind1]);
					continue;
				}
				if(udaljenosti[ind0][(int)stazaRod1[ind1]] > udaljenosti[ind0][(int)stazaRod2[ind2]])
				{
					elementi.Add(stazaRod2[ind2]);
					continue;
				}
			}
			Kromosom dijete = new Kromosom(elementi,this.izracunajDobrotu(elementi));
			return dijete;
		}
		
		/// <summary>
		/// Metoda koja radi PMX krizanje
		/// </summary>
		public Kromosom PartiallyMatchedCrossover (Kromosom Rod1, Kromosom Rod2, int tockaP1, int tockaP2)
		{
			ArrayList elementi1 = new ArrayList();
			ArrayList elementi2 = new ArrayList();

			for(int i = 0; i< this._brojGradova;i++)
			{
				elementi1.Add(null);
				elementi2.Add(null);
			}
			//dodavanje dijelova izmedju tocaka prekida
			for (int i = tockaP1+1; i <= tockaP2;i++)
			{
				elementi2[i] = Rod1.Chromosome[i];
				elementi1[i] = Rod2.Chromosome[i];
			}

			//generiranje ostatka kromosoma prema definiciji PMX krizanja
			for (int i = 0; i < this._brojGradova; i++)
			{
				if ((i > tockaP1) && (i <= tockaP2))
					continue;
				
				if (elementi1.Contains(Rod1.Chromosome[i]))
				{
					int index = Rod2.Chromosome.IndexOf(Rod1.Chromosome[i]);
					if (elementi1.Contains(Rod1.Chromosome[index]))
					{
						index = Rod2.Chromosome.IndexOf(Rod1.Chromosome[index]);
						elementi1[i] = Rod1.Chromosome[index];
					}
					else
					{
						elementi1[i] = Rod1.Chromosome[index];
					}
				}
				else
					elementi1[i] = (int)Rod1.Chromosome[i];
				
				if (elementi2.Contains(Rod2.Chromosome[i]))
				{
					int index = Rod1.Chromosome.IndexOf(Rod2.Chromosome[i]);
					if (elementi2.Contains(Rod2.Chromosome[index]))
					{
						index = Rod1.Chromosome.IndexOf(Rod2.Chromosome[index]);
						elementi2[i] = Rod2.Chromosome[index];
					}
					else
					{
						elementi2[i] = Rod2.Chromosome[index];
					}
					
				}
				else
					elementi2[i] = (int)Rod2.Chromosome[i];
			}
			//stvaranje djece
			Kromosom dijete1 = new Kromosom(elementi1,this.izracunajDobrotu(elementi1));
			Kromosom dijete2 = new Kromosom(elementi2,this.izracunajDobrotu(elementi2));
			//vracanje boljeg djeteta
			if (dijete1.DuljinaPuta < dijete2.DuljinaPuta)
				return dijete1;

			return dijete2;
		}
		/// <summary>
		/// Metoda koja radi GSX krizanje prema pseudokodu u dokumentaciji rada
		/// </summary>
		public Kromosom GreedySubtourCrossover (Kromosom Rod1, Kromosom Rod2)
		{
			ArrayList elementi = new ArrayList();
			bool fa = true, fb = true;
			Random rnd = new Random();
			
			int sluc = rnd.Next(this._brojGradova);
			int indR1 = Rod1.Chromosome.IndexOf(sluc);
			int indR2 = Rod2.Chromosome.IndexOf(sluc);
			elementi.Add(sluc);
			do
			{
				if (indR1 == 0)
					indR1 = 8;
				indR1 =(indR1 - 1) % this._brojGradova;
				indR2 =(indR2 + 1) % this._brojGradova;
				if (fa == true)
				{
					if(elementi.Contains(Rod1.Chromosome[indR1]))
					{
						fa = false;
					}
					else
					{
						elementi.Add(null);
						for(int i = elementi.Count-1; i>0;i--)
						{
							elementi[i] = elementi[i-1];
						}
						elementi[0] = Rod1.Chromosome[indR1];
					}
				}
				if (fb == true)
				{
					if(elementi.Contains(Rod2.Chromosome[indR2]))
					{
						fb = false;
					}
					else
					{
						elementi.Add(Rod2.Chromosome[indR2]);
					}
				}

			}while ((fa == true) || (fb == true));
	
			while (elementi.Count < this._brojGradova)
			{
				sluc = rnd.Next(this._brojGradova);
				if (elementi.Contains(sluc))
					continue;
				else
					elementi.Add(sluc);
			}
			
			Kromosom dijete = new Kromosom(elementi,this.izracunajDobrotu(elementi));
			return dijete;
		}

		/// <summary>
		/// Metoda koja radi jednostavnu mutaciju, zamjena dva grada unutar kromosoma
		/// </summary>
		public Kromosom JednostavnaMutacija (Kromosom zaMutirati)
		{
			ArrayList elementi = new ArrayList();
			foreach (int i in zaMutirati.Chromosome)
				elementi.Add(i);
			Random rnd = new Random();
			int gen1 = rnd.Next(this._brojGradova);
			int gen2 = rnd.Next(this._brojGradova); 
			int tmp = (int)elementi[gen1];
			elementi[gen1] = elementi[gen2];
			elementi[gen2] = tmp;
			
			Kromosom mutKromosom = new Kromosom(elementi,this.izracunajDobrotu(elementi));
			return mutKromosom;
		}
		/// <summary>
		/// Metoda koja radi Greedy Swap mutaciju, isto kao jednostavna samo na kraju ima
		/// usporedbu da li je dijete krace od roditelja, ako je vrati dijete inace roditelja
		/// </summary>
		public Kromosom GreedySwapMutacija (Kromosom zaMutirati)
		{
			ArrayList elementi = new ArrayList();
			foreach (int i in zaMutirati.Chromosome)
				elementi.Add(i);
			Random rnd = new Random();
			int gen1 = rnd.Next(this._brojGradova);
			int gen2 = rnd.Next(this._brojGradova); 
			int tmp = (int)elementi[gen1];
			elementi[gen1] = elementi[gen2];
			elementi[gen2] = tmp;
			
			Kromosom mutKromosom = new Kromosom(elementi,this.izracunajDobrotu(elementi));
			if (mutKromosom.DuljinaPuta <= zaMutirati.DuljinaPuta)
			{
				return mutKromosom;
			}
			return zaMutirati;
		}
		/// <summary>
		/// Metoda koja radi 2opt optimiranje ture
		/// </summary>
		public Kromosom _2opt(Kromosom zaOptimizirati)
		{
			double staraDuljina;
			
			do
			{
				staraDuljina = zaOptimizirati.DuljinaPuta;
				//za grad i provrti po svim ostalim gradovima da li je moguce skratiti turu
				for(int i = 0; i < this._brojGradova;i++)
				{
					int j = i+2;
					while (j < this._brojGradova)
					{	
						int indA = i;
						int indB = (i + 1) % this._brojGradova;
						int indC = j;
						int indD = (j + 1) % this._brojGradova;
						//provjera da li je zadovoljen uvjet za poboljsanje ture
						if (udaljenosti[(int)zaOptimizirati.Chromosome[indA]][(int)zaOptimizirati.Chromosome[indB]]+udaljenosti[(int)zaOptimizirati.Chromosome[indC]][(int)zaOptimizirati.Chromosome[indD]] > 
							udaljenosti[(int)zaOptimizirati.Chromosome[indA]][(int)zaOptimizirati.Chromosome[indC]]+udaljenosti[(int)zaOptimizirati.Chromosome[indB]][(int)zaOptimizirati.Chromosome[indD]])
						{
							int length = indC - indB;
							if(length < 0)
								length += zaOptimizirati.Chromosome.Count;
							length = length / 2 + 1;
							//okretanje indexa podture
							for(int l = 0; l < length; l++)
							{
								object temp = zaOptimizirati.Chromosome[indB];
								zaOptimizirati.Chromosome[indB] = zaOptimizirati.Chromosome[indC];
								zaOptimizirati.Chromosome[indC] = temp;
								if(++indB >= zaOptimizirati.Chromosome.Count)
									indB = 0;
								if(--indC < 0)
									indC = zaOptimizirati.Chromosome.Count-1;
							}						
							zaOptimizirati.DuljinaPuta = this.izracunajDobrotu(zaOptimizirati.Chromosome);
						}
						j++;	
					}
				}
			}while (staraDuljina > zaOptimizirati.DuljinaPuta);
			return zaOptimizirati;
		}

		/// <summary>
		/// Metoda koja vraca najboljeg clana populacije
		/// </summary>
		public Kromosom nadjiNajboljeg ()
		{
			Kromosom najbolji = (Kromosom)this.Populacija[0];
			foreach(Kromosom k in this.Populacija)
			{
				if (izracunajDobrotu(najbolji.Chromosome) > izracunajDobrotu(k.Chromosome))
					najbolji = k;
			}
			return najbolji;
		}

        #region ISerializable Members

        public Algoritam(SerializationInfo info, StreamingContext contex)
        {
            this.udaljenosti = info.GetValue("Udaljenost", typeof(double[][])) as double[][];
            this._brojGradova = info.GetInt32("BrojGradova");
            this._velPopulacije = info.GetInt32("VelicinaPopulacije");
            this._vjerSelekcije = info.GetInt32("VjerojatnostSeleckije");
            this._epsilon = info.GetDouble("Epsilon");
            inicijaliziraj(_velPopulacije);
        }

        public void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("Udaljenost", udaljenosti);
            info.AddValue("BrojGradova", _brojGradova);
            info.AddValue("VelicinaPopulacije", _velPopulacije);
            info.AddValue("VjerojatnostSeleckije", _vjerSelekcije);
            info.AddValue("Epsilon", _epsilon);            
        }

        #endregion
    }
}
