using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.Drawing.Drawing2D;
using System.Threading;
using CommonClass;
using System.Runtime.Serialization;

namespace TravelingSalesmanProblem_TSP.GeneticAlgorithmClass
{
    [Serializable]
    public partial class TSPGAManagerSimple : CommonClass.AlgorithmManager, ISerializable
    {
        public TSPGAManagerSimple(Panel _crtacaPloca, GraphicsPath _mousePath)
        {
            _parametri = new Parametri();
            InitializeComponent();
            CrtacaPloca = _crtacaPloca;
            CrtacaPloca.Paint += new PaintEventHandler(CrtacaPloca_Paint);
            mousePath = _mousePath;
        }

        public GraphicsPath mousePath;
        private GraphicsState gs;
        public void CrtacaPloca_Paint(object sender, PaintEventArgs e)
        {
            //Brush br = new SolidBrush(Color.Tomato);
            //e.Graphics.FillPath(br, mousePath);            
            if(crtaj)
            {
                if (this.crtajStazu == true)
                {
                    Pen blue = new Pen(Color.DarkBlue, 2);
                    for (int i = 0; i < this.brGradova; i++)
                    {
                        e.Graphics.DrawLine(blue, mousePath.PathPoints[((int)(najbolji.Chromosome[i])) * 4], mousePath.PathPoints[((int)(najbolji.Chromosome[(i + 1) % this.brGradova])) * 4]);                        
                    }                
                }                
            }
        }

        private Parametri _parametri;
        public Parametri Parametri
        {
            get { return _parametri; }
            set { _parametri = value; }
        }

        Thread pokreniAlgoritam;
        public override void StartAlgorithm()
        {
            SetGAParametars();

            this.pokreniAlgoritam = new Thread(new ThreadStart(Algorithm));
            this.pokreniAlgoritam.IsBackground = true;
            this.pokreniAlgoritam.Priority = System.Threading.ThreadPriority.Lowest;
            this.pokreniAlgoritam.Name = "TSP Algoritam";
            //pokretanje dretve
            this.pokreniAlgoritam.Start();
            this.Refresh();          
        }

        DateTime start;
        Algoritam TSPAlg;
        int brGradova;
        double[][] kord;
        Kromosom najbolji;
        bool crtajStazu = false;
        public Panel CrtacaPloca;
        DateTime stop;
        TimeSpan interval;
        bool crtaj = false;

        private void Algorithm()
        {
            Progress.Start(this.Location);
            crtaj = true;
            //uzimanje vremena za brojac
            start = System.DateTime.Now;
            int brojac = 0;
            Random rnd = new Random();
            int t1, t2, R;
            switch (this.Parametri.VrstaSelekcije)
            {
                //slucaj prirodne selekcije
                case 0:
                    this.TSPAlg = new Algoritam(this.Parametri.VelPopulacije, this.brGradova,
                        this.Parametri.VjerojatnostKrizanja, this.Parametri.Epsilon,
                        this.kord);

                    najbolji = this.TSPAlg.nadjiNajboljeg();
                    this.crtajStazu = true;
                    this.CrtacaPloca.Invalidate();
                    this.crtajStazu = false;

                    //izracunavanje tocaka prekida za slucaj PMX krizanja
                    t1 = rnd.Next(1, this.brGradova);

                    do
                    {
                        t2 = rnd.Next(1, this.brGradova);
                    } while (t1 <= t2);

                    //petlja algoritma
                    do
                    {
                        //ispis trenutnog statusa
                        //this.labelGen.Text = "Generacija/Nepromjenjeni: " + brojac.ToString() + ", " + "Najkraca staza: " + najbolji.DuljinaPuta.ToString();
                        if (this.crtajStazu)
                        {
                            this.CrtacaPloca.Invalidate();
                        }
                        this.crtajStazu = false;
                        this.TSPAlg.PrirodnaSelekcija();
                        R = this.Parametri.VelPopulacije * this.Parametri.VjerojatnostKrizanja / 100;
                        TSPAlg.Populacija.Clear();


                        //obavi krizanja i stavi djecu u TSPAlg.Populacija
                        for (int i = 0; i < R; i++)
                        {
                            int rod1 = rnd.Next(this.Parametri.VelPopulacije - R);
                            int rod2;
                            do
                            {
                                rod2 = rnd.Next(this.Parametri.VelPopulacije - R);
                            } while (rod1 == rod2);

                            Kromosom dijete = this.krizaj((Kromosom)TSPAlg.SortPop[rod1], (Kromosom)TSPAlg.SortPop[rod2], this.Parametri.VrstaKrizanja, t1, t2);
                            TSPAlg.Populacija.Add(dijete);
                        }
                        //radi mutaciju
                        R = this.TSPAlg.Populacija.Count * this.Parametri.VjerojatnostMutacije / 100;
                        for (int i = 0; i < R; i++)
                        {
                            ArrayList vecMutirani = new ArrayList();
                            int sluc;
                            do
                            {
                                sluc = rnd.Next(this.TSPAlg.Populacija.Count);
                            }
                            while (vecMutirani.IndexOf(sluc) != -1);

                            vecMutirani.Add(sluc);
                            this.TSPAlg.Populacija[sluc] = this.mutiraj((Kromosom)this.TSPAlg.Populacija[sluc], this.Parametri.VrstaMutacije);
                        }
                        foreach (object o in this.TSPAlg.SortPop)
                            this.TSPAlg.Populacija.Add(o);

                        //pronadji najboljeg
                        Kromosom tmp = this.TSPAlg.nadjiNajboljeg();
                        if (najbolji.DuljinaPuta > tmp.DuljinaPuta)
                        {
                            najbolji.DuljinaPuta = tmp.DuljinaPuta;
                            najbolji.Chromosome.Clear();
                            foreach (object o in tmp.Chromosome)
                                najbolji.Chromosome.Add(o);
                            this.crtajStazu = true;
                        }
                        //provjeri da li se treba zaustaviti s algoritmom
                        if ((this.Parametri.UvjetZaustavljanja == 0) && (this.Parametri.BrojGeneracija == brojac))
                        {
                            break;
                        }

                        if ((this.Parametri.UvjetZaustavljanja == 1))
                        {
                            if (this.Parametri.NepromjenjeniNajbolji == brojac)
                            {
                                break;
                            }
                            if (this.crtajStazu == true)
                                brojac = 0;
                        }

                        brojac++;
                        this.TSPAlg.SortPop.Clear();
                    } while (true);
                    this.crtajStazu = false;
                    break;
                //slucaj turnirske selekcije
                case 1:
                    TSPAlg = new Algoritam(this.Parametri.VelPopulacije, this.brGradova, this.kord);
                    najbolji = this.TSPAlg.nadjiNajboljeg();
                    this.crtajStazu = true;
                    this.CrtacaPloca.Invalidate();
                    this.crtajStazu = false;
                    //izracunavanje tocaka prekida za slucaj PMX krizanja
                    t1 = rnd.Next(1, this.brGradova);

                    do
                    {
                        t2 = rnd.Next(1, this.brGradova);
                    } while (t1 <= t2);

                    //petlja algoritma
                    do
                    {
                        //ispis statusa
                        //this.labelGen.Text = "Generacija/Nepromjenjeni: " + brojac.ToString() + ", " + "Najkraca staza: " + najbolji.DuljinaPuta.ToString();
                        if (this.crtajStazu)
                        {
                            this.CrtacaPloca.Invalidate();
                        }
                        this.crtajStazu = false;
                        ArrayList odabrani = new ArrayList();
                        //generiranje indexa djece kaj e biti mutirana	
                        ArrayList zaMutirati = new ArrayList();

                        R = System.Convert.ToInt32(this.Parametri.VelPopulacije * this.Parametri.VjerojatnostKrizanja / 100);
                        for (int j = 0; j < R * this.Parametri.VjerojatnostMutacije / 100; j++)
                        {
                            zaMutirati.Add(rnd.Next(R));
                        }
                        //radjenje turnirske selekcije
                        for (int j = 0; j < R; j++)
                        {

                            Kromosom najlosiji = this.TSPAlg.nadjiNajboljeg();

                            for (int i = 0; i < this.Parametri.VelTurnira; i++)
                            {
                                int broj = rnd.Next(this.Parametri.VelPopulacije);
                                if (odabrani.Contains(this.TSPAlg.Populacija[broj]))
                                {
                                    i--;
                                    continue;
                                }
                                odabrani.Add(this.TSPAlg.Populacija[broj]);
                                //ostaviti ili ne ovaj vece ili jednako!!!!!!!!!!!!!!!!!!!!!
                                if (((Kromosom)odabrani[i]).DuljinaPuta > najlosiji.DuljinaPuta)
                                    najlosiji = (Kromosom)odabrani[i];
                            }
                            odabrani.Remove(najlosiji);

                            this.TSPAlg.Populacija.Remove(najlosiji);
                            int rod1 = rnd.Next(odabrani.Count);
                            int rod2;
                            do
                            {
                                rod2 = rnd.Next(odabrani.Count);
                            } while (rod1 == rod2);

                            //napravi krizanje
                            najlosiji = this.krizaj((Kromosom)odabrani[rod1], (Kromosom)odabrani[rod2], this.Parametri.VrstaKrizanja, t1, t2);
                            //brojevi koji su mut % broja djece i ta djeca mutiraju
                            if (zaMutirati.Contains(j))
                            {
                                this.TSPAlg.Populacija.Add(this.mutiraj(najlosiji, this.Parametri.VrstaMutacije));
                            }
                            else
                                this.TSPAlg.Populacija.Add(najlosiji);

                            odabrani.Clear();
                        }
                        //nadji najboljeg
                        Kromosom tmp = this.TSPAlg.nadjiNajboljeg();
                        if (najbolji.DuljinaPuta > tmp.DuljinaPuta)
                        {
                            najbolji.DuljinaPuta = tmp.DuljinaPuta;
                            najbolji.Chromosome.Clear();
                            foreach (object o in tmp.Chromosome)
                                najbolji.Chromosome.Add(o);
                            this.crtajStazu = true;
                        }
                        //provjeri da li se treba zaustaviti s algoritmom
                        if ((this.Parametri.UvjetZaustavljanja == 0) && (this.Parametri.BrojGeneracija == brojac))
                        {
                            break;
                        }

                        if ((this.Parametri.UvjetZaustavljanja == 1))
                        {
                            if (this.Parametri.NepromjenjeniNajbolji == brojac)
                            {
                                break;
                            }
                            if (this.crtajStazu == true)
                                brojac = 0;
                        }

                        brojac++;
                    } while (true);
                    this.crtajStazu = false;
                    break;
            }
            //postavljanje sucelja nakon izvodjenja programa
            //this.labelGen.Text = najbolji.ToString();
            stop = System.DateTime.Now;
            interval = stop - start;
            //labelVrijeme.Text = "Vrijeme izvoenja: " + interval.ToString();
            //this.buttonStani.Visible = false;
            //this.buttonStart.Enabled = true;
            //this.buttonParametri.Enabled = true;
            //this.buttonReset.Enabled = true;
            this.crtajStazu = true;

            crtaj = false;

            if (OutputData.ContainsKey("Results"))
                OutputData.Remove("Results");

            OutputData.Add("Results", najbolji);

            Progress.End();
        }

        private void SetGAParametars()
        {

            brGradova = (int)this.InputData[InputFileParsers.KeyWordsTSP.brojGradova];
            kord = (double[][])this.InputData[InputFileParsers.KeyWordsTSP.matricaUdaljenosti];

            this.Parametri.VelPopulacije = System.Convert.ToInt32(this.textBoxVelicinaPopulacije.Text);
            if (this.radioButtonBrojGeneracija.Checked)
            {
                this.Parametri.UvjetZaustavljanja = 0;
                this.Parametri.BrojGeneracija = System.Convert.ToInt32(this.textBoxBrojGeneracija.Text);
            }
            else
            {
                this.Parametri.UvjetZaustavljanja = 1;
                this.Parametri.NepromjenjeniNajbolji = System.Convert.ToInt32(this.textBoxNepromjenjeni.Text);
            }
            this.Parametri.BrisanjeDuplikata = this.checkBoxBrisanjeDuplikata.Checked;

            if (this.radioButtonPrirodnaSelekcija.Checked)
            {
                this.Parametri.VrstaSelekcije = 0;
                this.Parametri.VjerojatnostKrizanja = (short)this.numericUpDownPrirodna.Value;
            }
            else
            {
                this.Parametri.VrstaSelekcije = 1;
                this.Parametri.VjerojatnostKrizanja = (short)this.numericUpDownTurnirniSelekcija.Value;
            }
            this.Parametri.Epsilon = System.Convert.ToDouble(this.textBoxEpsilon.Text);
            this.Parametri.VelTurnira = (short)this.numericUpDownTurniri.Value;

            if (this.radioButtonGSX.Checked)
            {
                this.Parametri.VrstaKrizanja = 0;
            }
            else
            {
                if (this.radioButtonGreedyCrossover.Checked)
                {
                    this.Parametri.VrstaKrizanja = 1;
                }
                else
                {
                    this.Parametri.VrstaKrizanja = 2;
                }
            }
            if (this.radioButtonJednostavna.Checked)
            {
                this.Parametri.VrstaMutacije = 0;
            }
            if (this.radioButtonGreedySwap.Checked)
            {
                this.Parametri.VrstaMutacije = 1;
            }
            if (this.radioButton2opt.Checked)
            {
                this.Parametri.VrstaMutacije = 2;
            }
            this.Parametri.VjerojatnostMutacije = (short)this.vjerojatnostMutacijeUpDown.Value;
        }

        private void buttonDefault_Click(object sender, EventArgs e)
        {
            this.Parametri.Default();
            IspisiParametre();
        }

        /// <summary>
        /// Ispis trenutnih parametara na ekran
        /// </summary>
        private void IspisiParametre()
        {
            this.textBoxVelicinaPopulacije.Text = this.Parametri.VelPopulacije.ToString();
            switch (this.Parametri.UvjetZaustavljanja)
            {
                case 0:
                    this.radioButtonBrojGeneracija.Checked = true;
                    break;
                case 1:
                    this.radioButtonNepromjenjeni.Checked = true;
                    break;
            }
            this.textBoxBrojGeneracija.Text = this.Parametri.BrojGeneracija.ToString();
            this.textBoxNepromjenjeni.Text = this.Parametri.NepromjenjeniNajbolji.ToString();
            this.checkBoxBrisanjeDuplikata.Checked = this.Parametri.BrisanjeDuplikata;
            //selekcija
            switch (this.Parametri.VrstaSelekcije)
            {
                case 0:
                    this.radioButtonPrirodnaSelekcija.Checked = true;
                    break;
                case 1:
                    this.radioButtonTurnirskaSelekcija.Checked = true;
                    break;
            }
            this.numericUpDownPrirodna.Value = this.Parametri.VjerojatnostKrizanja;
            this.textBoxEpsilon.Text = this.Parametri.Epsilon.ToString();
            this.numericUpDownTurnirniSelekcija.Value = this.Parametri.VjerojatnostKrizanja;
            this.numericUpDownTurniri.Value = this.Parametri.VelTurnira;

            //krizanje
            switch (this.Parametri.VrstaKrizanja)
            {
                case 0:
                    this.radioButtonGSX.Checked = true;
                    break;
                case 1:
                    this.radioButtonGreedyCrossover.Checked = true;
                    break;
                case 2:
                    this.radioButtonPMX.Checked = true;
                    break;
            }

            //mutacija
            switch (this.Parametri.VrstaMutacije)
            {
                case 0:
                    this.radioButtonJednostavna.Checked = true;
                    break;
                case 1:
                    this.radioButtonGreedySwap.Checked = true;
                    break;
                case 2:
                    this.radioButton2opt.Checked = true;
                    break;
            }
            this.vjerojatnostMutacijeUpDown.Value = this.Parametri.VjerojatnostMutacije;
        }

        //krizanje s odabranom vrstom krizanja
		private Kromosom krizaj (Kromosom rod1, Kromosom rod2, short tipKrizanja,int t1,int t2)
		{
			switch(tipKrizanja)
			{
				case 0:
					return TSPAlg.GreedySubtourCrossover(rod1,rod2);
				case 1:
					return TSPAlg.GreedyCrossover(rod1,rod2);
			}
			return TSPAlg.PartiallyMatchedCrossover(rod1,rod2,t1,t2);
			
		}
		//mutacija s odabranom vrstom mutacije
        private Kromosom mutiraj(Kromosom dijete, short tipMutacije)
        {
            switch (tipMutacije)
            {
                case 0:
                    return TSPAlg.JednostavnaMutacija(dijete);
                case 1:
                    return TSPAlg.GreedySwapMutacija(dijete);
            }
            return TSPAlg._2opt(dijete);
        }


        #region ISerializable Members

        public TSPGAManagerSimple(SerializationInfo info, StreamingContext contex)
            : base(info, contex)
        {
            InitializeComponent();

            textBoxBrojGeneracija.Text = info.GetString("textBoxBrojGeneracija");
            textBoxEpsilon.Text = info.GetString("textBoxEpsilon");
            textBoxNepromjenjeni.Text = info.GetString("textBoxNepromjenjeni");
            textBoxVelicinaPopulacije.Text = info.GetString("textBoxVelicinaPopulacije");
            radioButton2opt.Checked = info.GetBoolean("radioButton2opt");
            radioButtonBrojGeneracija.Checked = info.GetBoolean("radioButtonBrojGeneracija");
            radioButtonGreedyCrossover.Checked = info.GetBoolean("radioButtonGreedyCrossover");
            radioButtonGreedySwap.Checked = info.GetBoolean("radioButtonGreedySwap");
            radioButtonGSX.Checked = info.GetBoolean("radioButtonGSX");
            radioButtonJednostavna.Checked = info.GetBoolean("radioButtonJednostavna");
            radioButtonNepromjenjeni.Checked = info.GetBoolean("radioButtonNepromjenjeni");
            radioButtonPMX.Checked = info.GetBoolean("radioButtonPMX");
            radioButtonPrirodnaSelekcija.Checked = info.GetBoolean("radioButtonPrirodnaSelekcija");
            radioButtonTurnirskaSelekcija.Checked = info.GetBoolean("radioButtonTurnirskaSelekcija");
            checkBoxBrisanjeDuplikata.Checked = info.GetBoolean("checkBoxBrisanjeDuplikata");
            numericUpDownPrirodna.Value = info.GetInt32("numericUpDownPrirodna");
            numericUpDownTurniri.Value = info.GetInt32("numericUpDownTurniri");
            numericUpDownTurnirniSelekcija.Value = info.GetInt32("numericUpDownTurnirniSelekcija");
            vjerojatnostMutacijeUpDown.Value = info.GetInt32("vjerojatnostMutacijeUpDown");

            _parametri = info.GetValue("parametri", typeof(Parametri)) as Parametri;
        }

        void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
        {
            GetObjectData(info, context);

            info.AddValue("textBoxBrojGeneracija", textBoxBrojGeneracija.Text);
            info.AddValue("textBoxEpsilon", textBoxEpsilon.Text);
            info.AddValue("textBoxNepromjenjeni", textBoxNepromjenjeni.Text);
            info.AddValue("textBoxVelicinaPopulacije", textBoxVelicinaPopulacije.Text);
            info.AddValue("radioButton2opt", radioButton2opt.Checked);
            info.AddValue("radioButtonBrojGeneracija", radioButtonBrojGeneracija.Checked);
            info.AddValue("radioButtonGreedyCrossover", radioButtonGreedyCrossover.Checked);
            info.AddValue("radioButtonGreedySwap", radioButtonGreedySwap.Checked);
            info.AddValue("radioButtonGSX", radioButtonGSX.Checked);
            info.AddValue("radioButtonJednostavna", radioButtonJednostavna.Checked);
            info.AddValue("radioButtonNepromjenjeni", radioButtonNepromjenjeni.Checked);
            info.AddValue("radioButtonPMX", radioButtonPMX.Checked);
            info.AddValue("radioButtonPrirodnaSelekcija", radioButtonPrirodnaSelekcija.Checked);
            info.AddValue("radioButtonTurnirskaSelekcija", radioButtonTurnirskaSelekcija.Checked);
            info.AddValue("checkBoxBrisanjeDuplikata", checkBoxBrisanjeDuplikata.Checked);
            info.AddValue("numericUpDownPrirodna", numericUpDownPrirodna.Value);
            info.AddValue("numericUpDownTurniri", numericUpDownTurniri.Value);
            info.AddValue("numericUpDownTurnirniSelekcija", numericUpDownTurnirniSelekcija.Value);
            info.AddValue("vjerojatnostMutacijeUpDown", vjerojatnostMutacijeUpDown.Value);

            info.AddValue("parametri", _parametri);
        }

        #endregion
    }
}

