using System;
using System.Collections.Generic;
using System.Text;
using CommonClass;
using System.Xml;
using System.IO;
using System.Drawing;
using System.Runtime.Serialization;

namespace TravelingSalesmanProblem_TSP.InputFileParsers
{
    public enum KeyWordsTSP
    {
        xmlDocument,
        brojGradova,
        tocke,
        matricaUdaljenosti,
        sX,
        sY
    };

    [Serializable]
    class Xml : IInputParser , ISerializable
    {
        private string _textProblema;
        [NonSerialized]private XmlDocument _document;
        private int _brojGradova;
        private List<PointF> _pointList;
        private double[][] _matricaUdaljenosti;
        private float sX;
        private float sY;

        public Xml(string inputText)
        {
            _textProblema = inputText;
            if (!string.IsNullOrEmpty(inputText))
            {
                _document = new XmlDocument();
                _document.LoadXml(inputText);
            }
        }

        public Xml(StreamReader inputFile)
        {
            _textProblema = inputFile.ReadToEnd();
            if (!string.IsNullOrEmpty(_textProblema))
            {
                _document = new XmlDocument();
                _document.Load(inputFile);
            }
            
        }

        #region IInputParser Members

        public Dictionary<object, object> GetResult
        {
            get 
            {
                Dictionary<object, object> results = new Dictionary<object, object>();
                //results.Add(KeyWordsTSP.xmlDocument, _document);
                results.Add(KeyWordsTSP.brojGradova, _brojGradova);
                results.Add(KeyWordsTSP.tocke, _pointList);
                results.Add(KeyWordsTSP.matricaUdaljenosti, _matricaUdaljenosti);
                results.Add(KeyWordsTSP.sX, sX);
                results.Add(KeyWordsTSP.sY, sY);

                return results;
            }
        }

        public string SetInputText
        {
            set 
            {
                if (!string.IsNullOrEmpty(value))
                {
                    _document = new XmlDocument();
                    _document.LoadXml(value);
                }
                _textProblema = value;                 
            }
        }

        private string GetValue(string section, string name)
        {
            try
            {
                return this._document.DocumentElement.SelectSingleNode(section + "/" + name).InnerText;
            }
            catch (Exception ex)
            {
                return ex.Message;
            }
        }

        public void Start()
        {
            _pointList = new List<PointF>();
            _brojGradova = int.Parse(GetValue("info", "brojGradova"));

            string[] str = GetValue("tocke", "t0").Split(' ');
            PointF t = new PointF((float)System.Double.Parse(str[0]), (float)System.Double.Parse(str[1]));
            _pointList.Add(t);
            PointF min = new PointF(t.X, t.Y);
            PointF max = new PointF(t.X, t.Y);
            //vadjenje koordinata i odredjivanje min i max vrijednosti
            for (int i = 1; i < this._brojGradova; i++)
            {
                str = GetValue("tocke", "t" + i.ToString()).Split(' ');
                t.X = (float)System.Double.Parse(str[0]);
                t.Y = (float)System.Double.Parse(str[1]);
                _pointList.Add(t);
                if (min.X > t.X)
                    min.X = t.X;
                if (min.Y > t.Y)
                    min.Y = t.Y;
                if (max.X < t.X)
                    max.X = t.X;
                if (max.Y < t.Y)
                    max.Y = t.Y;
            }
            //stvaranje matrice udaljenosti
            _matricaUdaljenosti = new double[_brojGradova][];
            for (int i = 0; i < _brojGradova; i++)
                _matricaUdaljenosti[i] = new double[_brojGradova];

            for (int i = 0; i < _brojGradova - 1; i++)
            {
                PointF gradI = (PointF)_pointList[i];
                for (int j = i + 1; j < _brojGradova; j++)
                {
                    PointF gradJ = (PointF)_pointList[j];

                    double udaljenost = Math.Sqrt(Math.Pow(gradI.X - gradJ.X, 2) + Math.Pow(gradI.Y - gradJ.Y, 2));
                    _matricaUdaljenosti[i][j] = udaljenost;
                    _matricaUdaljenosti[j][i] = udaljenost;
                }
                _matricaUdaljenosti[i][i] = 0;
            }
            //					if (min.X < 0)
            //					{
            for (int i = 0; i < _pointList.Count; i++)
            {
                t.X = ((PointF)_pointList[i]).X - min.X;
                t.Y = ((PointF)_pointList[i]).Y;

                _pointList.Remove(_pointList[i]);
                _pointList.Insert(i, t);
            }
            //					}

            //					if (min.Y < 0)
            //					{
            for (int i = 0; i < _pointList.Count; i++)
            {
                t.X = ((PointF)_pointList[i]).X;
                t.Y = ((PointF)_pointList[i]).Y - min.Y;

                _pointList.Remove(_pointList[i]);
                _pointList.Insert(i, t);
            }
            //					}
             sX = max.X - min.X;
             sY = max.Y - min.Y;           
        }
        
        #endregion

        #region ISerializable Members

        public Xml(SerializationInfo info, StreamingContext contex)
        {
            this._textProblema = info.GetString("TextProblem");
            this._brojGradova = info.GetInt32("BrojGradova");
            this._pointList = info.GetValue("PointList", typeof(List<PointF>)) as List<PointF>;
            this._matricaUdaljenosti = info.GetValue("MatricaUdaljenosti", typeof(double[][])) as double[][];
            this.sX = (float)info.GetValue("SX", typeof(float));
            this.sY = (float)info.GetValue("SY", typeof(float));
        }

        public void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("TextProblem", _textProblema);
            info.AddValue("BrojGradova", _brojGradova);
            info.AddValue("PointList", _pointList);
            info.AddValue("MatricaUdaljenosti", _matricaUdaljenosti);
            info.AddValue("SX", sX);
            info.AddValue("SY", sY);
        }

        #endregion
    }
}
