using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CommonClass;
using System.Reflection;
using System.Runtime.Serialization;

namespace TravelingSalesmanProblem_TSP
{
    [Serializable]
    public partial class TravelingSalesmanProblem_TSP : CommonClass.WorkingSession, ISerializable
    {

        protected override void InitializeComponentAndData()
        {
            InitializeComponent();
            mousePath = new System.Drawing.Drawing2D.GraphicsPath();
        }

        public TravelingSalesmanProblem_TSP()
        {
            InitializeComponentAndData();
        }

        public TravelingSalesmanProblem_TSP(Dictionary<string, object> extendData, ImplementedAlgorithmType algorithmType)
            : base(extendData)
        {
            switch (algorithmType)
            {
                case ImplementedAlgorithmType.GeneticAlgorithm:
                    algorithmManager = new GeneticAlgorithmClass.TSPGAManagerSimple(this.CrtacaPloca, this.mousePath);
                    break;
                case ImplementedAlgorithmType.SI:
                    break;
                case ImplementedAlgorithmType.error:
                    throw new Exception("Nepostojei algoritam!");
            }

            RegisterFinishedAlgorithmEvent();
            ShowAlgorithmManager();            
        }

        protected override void RegisterFinishedAlgorithmEvent()
        {
            algorithmManager.FinishedAlgorithm += new FinishedAlgorithmHandler(algorithmManager_FinishedAlgorithm);
        }

        void algorithmManager_FinishedAlgorithm()
        {
            
        }

        protected override DataTable SetInputFileTypes(DataTable data, string description, string name)
        {
            DataRow row = data.NewRow();

            row[description] = "XML Graf";
            row[name] = "xml";

            data.Rows.Add(row);

            return data;
        }

        protected override DataTable SetOutputFileTypes(DataTable data, string description, string name)
        {
            DataRow row = data.NewRow();

            row[description] = "WriteResults";
            row[name] = "WriteResults";

            data.Rows.Add(row);

            return data;
        }

        protected override string SetupInputFileExtension()
        {
            return "XML Graf (*.xml) |*.xml";
        }

        protected override CommonClass.IInputParser ProblemDefinition(string inputText, string selectedExtension)
        {
            rtbProblem.Text = inputText;
            rtbProblem.BringToFront();
            return DynamicalCall.GetInputParser(selectedExtension, inputText, Assembly.GetExecutingAssembly());
        }

        protected override IOutputParser SetOutputParser(string outputParserType)
        {
            return DynamicalCall.GetOutputParser(outputParserType, Assembly.GetExecutingAssembly());
        }

        protected override IInputParser SetInputParser(string inputParserType)
        {
            return DynamicalCall.GetInputParser(inputParserType, "", Assembly.GetExecutingAssembly());
        }

        protected override string SetInputText()
        {
            return rtbProblem.Text;
        }

        protected override string SetupOuputFileExtension()
        {
            return "Results | *.out";
        }

        protected override void MakeNewProblem()
        {
            this.CrtacaPloca.Invalidate();
            this.rtbProblem.Clear();
            this.rtbProblem.BringToFront();
            this.CrtacaPloca.BringToFront();
        }

        protected override string ProblemToString()
        {
            return rtbProblem.Text;
        }

        private System.Drawing.Drawing2D.GraphicsPath mousePath;
        protected override void AfterSuccessCheckSyntaks()
        {
            mousePath.Reset();
            foreach (PointF p in (List<PointF>)_inputParserFormat.GetResult[InputFileParsers.KeyWordsTSP.tocke])
            {
                Rectangle prav = new Rectangle((int)(5 + this.CrtacaPloca.Size.Width * 0.97 * p.X / (float)_inputParserFormat.GetResult[InputFileParsers.KeyWordsTSP.sX])
                                                , (int)(5 + this.CrtacaPloca.Size.Height * 0.97 * p.Y / (float)_inputParserFormat.GetResult[InputFileParsers.KeyWordsTSP.sY])
                                                , 5
                                                , 5);
                mousePath.AddRectangle(prav);
                CrtacaPloca.Invalidate();                
            }
            CrtacaPloca.BringToFront();

            CrtacaPloca.Refresh();

            tabResulats.SelectTab(1);
        }

        protected override void BeforeCheckSyntaks()
        {            
            CrtacaPloca.Invalidate();            
            mousePath.Reset();
        }

        private void CrtacaPloca_Paint(object sender, PaintEventArgs e)
        {
            Brush br = new SolidBrush(Color.Tomato);
            e.Graphics.FillPath(br, mousePath);            
        }

        #region ISerializable Members

        public TravelingSalesmanProblem_TSP(SerializationInfo info, StreamingContext contex)
            :base(info, contex)
        {
            this.rtbProblem.Text = info.GetValue("problemDefinition", typeof(string)) as string;
            RegisterFinishedAlgorithmEvent();
            ShowAlgorithmManager();

            if (algorithmManager is GeneticAlgorithmClass.TSPGAManagerSimple)
            {
                (algorithmManager as GeneticAlgorithmClass.TSPGAManagerSimple).CrtacaPloca = CrtacaPloca;
                (algorithmManager as GeneticAlgorithmClass.TSPGAManagerSimple).CrtacaPloca.Paint += new PaintEventHandler((algorithmManager as GeneticAlgorithmClass.TSPGAManagerSimple).CrtacaPloca_Paint);
                (algorithmManager as GeneticAlgorithmClass.TSPGAManagerSimple).mousePath = mousePath;
            }

            CheckSyntaks();
        }

        void ISerializable.GetObjectData(SerializationInfo info, StreamingContext context)
        {
            GetObjectData(info, context);

            info.AddValue("problemDefinition", rtbProblem.Text);            
        }

        #endregion
    }
}

